##################### 

# Transcriptome analysis

#####################


# Libraries needed for analysis

library(ShortRead)
library(gtools)
library(xlsx)
library(gplots)
library(viridis)
library(readr)
library(randomForest)
library(ROCR)
library(MASS)
library(ggplot2)
library(class)
library(Biobase)
library(GEOquery)
library(limma)
library(extrafont)
library(gridExtra)

## Global variables

# Number of errors allowed in barcodes
BCErrorAllowance <- 0
# Levenshtein distance allowed for bead identifier matching
ManualEdit <- 2
# Whether or not quality thresholds for location identification are applied
QualityTrim <- TRUE
# Whether or not a manual threshold for initial Levenshtein distance is used
BeadCutoff <- NULL

# Source folder paths
sourcefolder <- "Source for code"
filefolder <- "Source for sequencing data"
setwd(filefolder)

# Location barcodes used (6x column, 6x row)
BarcodesSeqsColumn <- c("GACCATAAAAAAAAAA","GAATGTAAAAAAAAAA","AGCAAGAAAAAAAAAA","GTCTCTAAAAAAAAAA","TGCTTGAAAAAAAAAA","CTCGCGAAAAAAAAAA")
BarcodesSeqsRow <- c("GAATTGAAAAAAAAAA","TTAGACAAAAAAAAAA","ACGACTAAAAAAAAAA","AATACGAAAAAAAAAA","AATTTGAAAAAAAAAA","GTGAACAAAAAAAAAA")

# Names for location barcodes
BarcodeLabelsColumn <- c("X1-A","X2-B","X3-C","X4-D","X5-E","X6-F")
BarcodeLabelsRow <- c("Y1-1","Y2-2","Y3-3","Y4-4","Y5-5","Y6-6")

# Package for location barcodes
BarcodesColumn <- cbind(BarcodesSeqsColumn, BarcodeLabelsColumn)
BarcodesRow <- cbind(BarcodesSeqsRow, BarcodeLabelsRow)

## Define functions used

# Function to obtain mode of a vector
ModeThenMin <- function(x) 
{
  ux <- unique(x) 
  min(ux[tabulate(match(x, ux))==tabulate(match(x, ux))[which.max(tabulate(match(x, ux)))]])
}

# Function to group bead identifier sequences

# Chooses a subset of beads that contribute the majority of total reads, 
# clusters those beads by Levenshtein distance within a certain height, 
# groups remainder of beads with low reads with Levenshtein distance clusters
BeadIdentifierandMismatchExtractionwUMIs <- function(R1Reads, FileName=NULL, MinimumAppearance=NULL,
                                                     BILength=NULL, CutHeight=NULL)
{
  # Typical bead identifier is length 11, 12 nucleotide is a common error with beads
  if(is.null(BILength)) {BILength <- 11}
  # Minimum number of reads as a function of the total reads 
  if(is.null(MinimumAppearance)) {MinimumAppearance=length(R1Reads)/1000}

  if(is.null(FileName)) {FileName <- "Default"}
  # Loose threshold for Levenshtein distance if unspecified
  if(is.null(CutHeight)) {CutHeight <- 4}

  # Isolate the bead identifiers of each read
  BeadIdentifiers <- subseq(R1Reads, 1, BILength)
  
  # Isolate the UMIs of each read
  UMIs <- subseq(R1Reads, 13, 20)
  
  # Identify all unique bead identifiers by number of appearances
  GroupedBI <- table(BeadIdentifiers)

  # Identify all unique bead identifiers by sequence
  GroupedBINames <- names(GroupedBI)
  
  # Identify the most commonly appearing bead identifiers by number of appearances
  RankOrderedBI <- sort(GroupedBI, decreasing=TRUE)
  
  # Alternative method to choose minimum number of beads user for clustering
  # Use clustering on beads that contribute 66% of total reads
  if(is.null(MinimumAppearance)) 
  {
    MinimumAppearance <- RankOrderedBI[which(cumsum(RankOrderedBI)>length(R1Reads)*2/3)[1]]
  }
  
  # Most commonly appearing bead identifiers as DNA
  AllDNABIs <- DNAStringSet(names(RankOrderedBI))
  
  # Remove all beads less frequent than MinimumAppearance for top hits list
  ThresholdedBI <- RankOrderedBI[RankOrderedBI>MinimumAppearance]
  
  # Collect beads with few appearances
  LeftoverBI <- DNAStringSet(names(RankOrderedBI[RankOrderedBI<=MinimumAppearance]))
  
  # Identify the most commonly appearing bead identifiers by number of appearances
  PreliminarySequencestoFind <- DNAStringSet(names(ThresholdedBI))
   
  # Number of BIs in initial clustering step
  PreliminaryAppearancesAboveThreshold <- length(PreliminarySequencestoFind)
  
  # Matrix of Levenshtein distances between top hits
  PreliminaryDistanceMap <- adist(PreliminarySequencestoFind)
  
  # Clustering of Levenshtein distances
  PreliminaryDendrogram <- hclust(as.dist(PreliminaryDistanceMap))
  
  plot(PreliminaryDendrogram, labels=PreliminarySequencestoFind, hang=-1, main=paste0(FileName, CutHeight))
  
  # If arbitrary cut height is necessary based on distribution of bead Levenshtein distances
  if(CutHeight=="MANUAL")
  {CutHeight <- readline("CutHeight?")
  print("OK")}
  
  # Cut dendrogram at specified height and group numbers assigned to each cluster within the specified height
  GroupNumbers <- cutree(PreliminaryDendrogram, h=CutHeight)
  
  # Assign each group to one barcode
  RepresentativeBarcodes <- PreliminarySequencestoFind[which(!duplicated(GroupNumbers))]
  
  ## Heatmap of bead distance
  
  # Plot the heatmap of bead Levenshtein distances
  heatmap.2(PreliminaryDistanceMap, Rowv=as.dendrogram(PreliminaryDendrogram), 
            Colv=as.dendrogram(PreliminaryDendrogram), col=inferno(BILength, direction=-1), 
            key=TRUE, trace="none", main=paste0(FileName, CutHeight))
  
  
  
  # Placeholder for Bead Identifiers
  GroupNumberOutput <- vector("character", length(R1Reads))
  
  #Placeholder for BeadID Sequences
  BISequence <- vector("character", length(R1Reads))
  
  # This stores the mismatched sequences
  BeadIdentifierContainer <- vector("character", length(R1Reads))
 
  # Initialize the final list output
  SequenceOutput <- PreliminarySequencestoFind
  
  # Progress tracking
  ProgressCounter <- 0
  
  # Count the total BIs starting with the most popular ones
  CountList <- ThresholdedBI
  
  # Go through the list of beads with few reads
  for (n in 1:length(LeftoverBI))
  {
    # Progress counter
    if (length(LeftoverBI)>10){
      if(n>ProgressCounter*length(LeftoverBI)/10)
      {
        
        cat(ProgressCounter,"0% Done; ", sep = "")
        ProgressCounter <- ProgressCounter+1
      }
    }
    
    #Place leftover things in bins based on the preliminary list
    Hit <- which(adist(LeftoverBI[n], PreliminarySequencestoFind)<=2)
    
    # If there is a clustered bead that is similar to one of the bead identifiers with few reads, record the hit
    if (!isEmpty(Hit)) 
    {
      # Record the sequence
      SequenceOutput <- append(SequenceOutput, LeftoverBI[n])
      
      # Place the hit in the bin that it matches the most elements of
      GroupNumbers <- append(GroupNumbers, ModeThenMin(GroupNumbers[Hit]))
      
      # Record the number of reads contributed
      CountList <- append(CountList, GroupedBI[GroupedBINames==LeftoverBI[n]])
    }
  }
  print("100% Done")
  
  # Assign each cluster of similar beads the name of the most popular bead
  GroupNames <- RepresentativeBarcodes[GroupNumbers]
  
  # Record the lists for downstream processing and sequencing
  write.csv(data.frame(SequenceOutput, GroupNames, GroupNumbers, CountList), 
            file=paste0(FileName, "Grouping.csv"), row.names = FALSE)
  write.table(as.character(unique(GroupNames)), paste0(FileName, "cellbarcodes.csv"), 
              quote=FALSE, row.names=FALSE, col.names=FALSE)
  
  # Look at the list of similar bead identifier clusters
  for(n in 1:max(GroupNumbers))
  {
    # Find all hits that match to a group number
    GroupHits <- which(GroupNumbers %in% n)
    
    # Find the sequences of those hits
    GroupList <- SequenceOutput[GroupHits]
    
    # Assign a common name to all similar bead identifiers
    GroupNumberOutput <- replace(GroupNumberOutput, which(BeadIdentifiers %in% GroupList), 
                                 paste("Group#", n, sep=""))
    
  }
  
  # Unassigned beads are labeled VOID
  GroupNumberOutput <- replace(GroupNumberOutput,GroupNumberOutput=="","VOID")
  
  # Output data
  BeadIdentifierandMismatchExtractionwUMIs <- data.frame(GroupNumberOutput, BeadIdentifiers, UMIs)
  
}


# Function to analyze location barcodes only

# Searches through reads for location barcodes and labels the read with the location
BarcodeExtraction <- function(R2Reads, BarcodeReferences, ErrorsAllowed)
{
  # Removes common errors from transposons or truncation products
  R2Reads <- gsub(".*TGTATAAGAGACAG", "", R2Reads)
  R2Reads[nchar(R2Reads)<20]="AAAAAAAAAAAAAAAAAAAA"
  
  # Counts the number of possible barcodes
  NumberofBarcodes <-  nrow(BarcodeReferences)

  # Uses first base pairs equal to length of reference barcodes (depends on number of A's inserted for accuracy)  
  BarcodeRegion <- subseq(R2Reads,1,nchar(BarcodeReferences[1]))

  # Counts the number of unique sequences that match the length of the barcode sequence used in the reference table
  UniqueBarcodes <- names(table(BarcodeRegion))
  
  # Placeholder for the barcode Labels
  Container <- vector("character",length(R2Reads))
  
  # For each barcode, identifies reads assigned to that barcode to be matched to the bead identifier
  for (n in 1:NumberofBarcodes)
  {
    
    # Determine which reads match barcodes with some errors allowed
    Hits=vcountPattern(BarcodeReferences[n,1],UniqueBarcodes,
                       max.mismatch=ErrorsAllowed,with.indels=TRUE)
    
    # Extracts the barcodes based on index of the hits in the list of unique barcodes that match
    HitIdentity = UniqueBarcodes[Hits*1:length(Hits)]
    
    # Label those reads in the original read order with barcode title
    Container <- replace(Container,which(BarcodeRegion %in% HitIdentity),BarcodeReferences[n,2])
  }
  
  # Reads not assigned to a location are labeled VOID
  Container <- replace(Container,Container=="","VOID")
  
  # Data output
  BarcodeExtraction <- data.frame(Container)
  
}

# Function to correlate each bead identifier to its likely location


# Search through bead identifiers, collect all the location barcodes for each bead identifier, 
# and assign a location to each bead identifier

BIBarcodeCorrelation <- function(BeadIdentifierDataFrame, BarcodeLabelDataframe, Grouping, ColumnorRow=NULL, FileName=NULL)
{
  # Default labels if none given
  if(is.null(ColumnorRow)) {ColumnorRow <- ""}
  if(is.null(FileName)) {FileName <- ""}
  
  # BI number of bead identifiers to search through
  GroupNumberLabels <- BeadIdentifierDataFrame$GroupNumberOutput
  
  # BI sequence of unique bead identifiers
  GroupNumberRepresentatives <- Grouping$GroupNames[!duplicated(Grouping$GroupNumbers)]
  
  # Vector of location barcodes
  BarcodeLabels <- BarcodeLabelDataframe$Container
  
  # How many BIs there are to test
  GroupNumber <- length(unique(Grouping$GroupNumbers))
  
  # UMI of each bead identifier for quality check
  BeadIdentifierUMIs <- BeadIdentifierDataFrame$UMIs
  
  #Initialized for most common barcodes for each BI
  MaxBarcode <- vector("character", GroupNumber)
  
  #Initialized for BI of each BI
  BarcodeBISequence <- vector("character", GroupNumber)
  
  #Initialized for proportion of barcodes contributed by dominant barcode
  MaxBarcodeQuality <- vector("numeric", GroupNumber)
  
  #Initialized for barcode reads over total reads
  TotalReadsUsedForBarcodes <- vector("numeric", GroupNumber)
  
  #Initialized for number of UMI collapsed barcodes
  TotalCollapsedBarcodes <- vector("numeric", GroupNumber)
  
  #Initialized for most common barcodes for each BI
  SecondBarcode <- vector("character", GroupNumber)
  
  #Initialized for most common barcodes for each BI
  SecondBarcodeQuality <- vector("numeric", GroupNumber)
  
  
  ReadTotal <- vector("numeric", GroupNumber)
  
  Container <- vector("character", GroupNumber)
  
  UMICountList <- vector("numeric", GroupNumber)
  
  FullSummary <- data.frame()
  
  # For each bead identifier
  for (n in 1:GroupNumber)
    
  {
    
    # Look at each BI name
    BITarget <- paste0("Group#",n)
    
    #Look for Groups and grab their index from master list
    IndexedHits <- which(GroupNumberLabels == BITarget)
    
    #Total reads for a Group
    ReadTotal[n] <- length(IndexedHits)
    
    #All the barcodes that share a BI
    TargetBarcode <- BarcodeLabels[IndexedHits]
    
    # Indexes of BI hits that can be assigned to a barcode
    RealIndexedHits <- IndexedHits[TargetBarcode!="VOID"]
    
    # Reads for a BI that are barcodes
    BarcodeOccupancy <- length(RealIndexedHits)
    
    # Everything for RealIndexedHits works on the subsampling
    
    # All the UMIs for the reads containing a BC
    UMIList <- BeadIdentifierUMIs[RealIndexedHits]
    
    # Barcodes of reads
    BarcodeList <- BarcodeLabels[RealIndexedHits]

    # Collapse reads by UMIs
    CollapsedUMINoiseFilteredBarcodes <- BarcodeList[!duplicated(UMIList)]

    #List of BCs that pass quality check
    CollapsedUMIBarcodeCoun <- sort(table(CollapsedUMINoiseFilteredBarcodes), decreasing=TRUE)
    

    # What is the most frequence barcode
    MaxBarcode[n] <- names(CollapsedUMIBarcodeCount)[1]
    
    # How frequent is the most frequent barcode
    MaxBarcodeQuality[n] <- CollapsedUMIBarcodeCount[1]/sum(CollapsedUMIBarcodeCount)*100
    
    # How often does each barcode appear
    FullSummaryElement <- paste0(CollapsedUMIBarcodeCount, "-", names(CollapsedUMIBarcodeCount))
    
    # Assemble list of appearances for barcodes
    if (n==1)
    {FullSummary <- FullSummaryElement}
    else
    {FullSummary <- rbind(FullSummary,FullSummaryElement)}
    
    # How many total barcode reads are there after UMI collapse
    TotalCollapsedBarcodes[n] <- sum(CollapsedUMIBarcodeCount)
    
    # How much of the total reads is location barcodes
    TotalReadsUsedForBarcodes[n] <- BarcodeOccupancy/ReadTotal[n]
    
  }
  
  
  # Assemble for each bead sequence, its group number, the most frequent location barcode, how often it appears,
  # the total number of barcodes, how often a read from the bead identifier is a barcode, and the total reads
  BI = GroupNumberRepresentatives
  Bead = c(1:GroupNumber)
  BC1 = MaxBarcode
  Q1 = format(MaxBarcodeQuality, digits = 2)
  T1 = TotalCollapsedBarcodes
  R1 = format(TotalReadsUsedForBarcodes, digits = 2)
  S1 = ReadTotal
  
  
  output <- data.frame(I(BI), Bead, BC1, Q1, T1, R1, S1, stringsAsFactors = FALSE)
  
  names(output) <- paste0(ColumnorRow, c("BeadIdentifier", "BeadGroup", "Location",
                                         "QualityScore", "UniqueBarcodes", "ReadsasBarcodes",
                                         "TotalReads"))

  # Produce list of all barcodes for each bead identifier after UMI collapse
  rownames(FullSummary) <- paste0("Group",1:GroupNumber)
  
  write.csv(FullSummary, file=paste0(FileName, ColumnorRow, "FullSummary.csv"))
  
  # Data output
  BIBarcodeCorrelation <- output
}





# Function to use the compressed bead identifiers to rewrite the bead identifiers and cluster the reads by bead

ShortReadGroupNumberEdit <- function(FastqInput, GroupingDictionary, BILength=NULL, FileName=NULL)
{
  # Checks the length of bead identifier to be rewritten
  if(is.null(BILength)) {BILength <- 11}
  
  # Extracts information from fastq files
  ShortReadInput <- sread(FastqInput)
  IDInput <- FastqInput@id
  ShortReadQuality <- quality(FastqInput)
  
  # Extracts information from grouping given by GroupingDictionary
  Reference <- GroupingDictionary$SequenceOutput
  Replacements <- GroupingDictionary$GroupNames
  GroupNumberLabel <- GroupingDictionary$GroupNumbers
  
  # Counter
  ProgressCounter <- 0
  
  # How many groups does the grouping dictionary provide
  TotalGroups <- length(table(GroupNumberLabel))
  
  # Isolates the bead identifiers of all the sequences
  BeadIdentifiers <- subseq(ShortReadInput,start=1,end=BILength)
  
  # Saves the rest of each sequence after the bead identifier
  Remainder <- subseq(ShortReadInput, start=BILength+1, end=nchar(ShortReadInput))
  
  # Saves the entire unmodified sequence input
  CollapsedBeadIdentifiers <- ShortReadInput

  # For every group, rewrite the bead identifiers to be the same within the group
  for(Counter in 1:TotalGroups){
    # Progress counter
    if (TotalGroups>10){
      if(Counter>ProgressCounter*TotalGroups/10)
      {
        
        cat(ProgressCounter,"0% Done; ", sep = "")
        ProgressCounter <- ProgressCounter+1
      }
    }
    
    # For each group, find the sequence that will represent that group
    ReplacementRepresentative <- Replacements[match(Counter, GroupNumberLabel)]
    
    # Find every sequence in the sequencing data that should be in the group
    ReferencestoReplaceinGroup <- Reference[GroupNumberLabel==Counter]
    
    # Extract the index of those sequences
    ReplacementIndex <- BeadIdentifiers %in% ReferencestoReplaceinGroup
    
    # Save the remainder of those sequences
    RemainderSequence <- Remainder[ReplacementIndex]
    
    # Create the replacements for those sequences by changing all bead identifiers in that group to the representative
    ReconstructedRead <- DNAStringSet(paste0(ReplacementRepresentative,RemainderSequence))
    
    # Reconstruct the read by replacing every read from that group with the replacement reads with a single bead identifier
    CollapsedBeadIdentifiers <- replace(CollapsedBeadIdentifiers, ReplacementIndex, ReconstructedRead)
    
    
  }  
  print("100% Done")
  
  # Reconstruct the fastq file
  output <- ShortReadQ(sread=CollapsedBeadIdentifiers, id=IDInput, quality=ShortReadQuality)
  
  ShortReadGroupNumberEdit <- output
  writeFastq(output, paste0(FileName,"GroupEditedR1.fastq.gz"))
}



# Function to use the location tables to compress bead identifiers by shared location

LocationCompressionFunction <- function(FastqInput, CorrelationFile, BILength=NULL, FileName=NULL)
{
  # Default bead identifier length
  if(is.null(BILength)) {BILength <- 11}
  
  # Isolate sequencing data components
  ShortReadInput <- FastqInput@sread
  IDInput <- FastqInput@id
  ShortReadQuality <- FastqInput@quality
  
  # Extract bead sequences and both locations from the correlation file
  BeadDNASequence <- as.character(CorrelationFile$ColumnBeadIdentifier)
  ColumnLocations <- as.factor(CorrelationFile$ColumnLocation)
  RowLocations <- as.factor(CorrelationFile$RowLocation)
  
  # How many beads are there
  ExtractedBeads <- nrow(CorrelationFile)
  
  # Initialize 
  OccupiedLocations <- vector("character", ExtractedBeads)
  LocationReference <- vector("character", ExtractedBeads)
  
  # For each bead, 
  for (n in 1:ExtractedBeads)
  {
    # Rename beads by their location on chip
    Location <- paste0(ColumnLocations[n],RowLocations[n])
    
    # Initialize the list
    if(n==1)
    {
      OccupiedLocations[n] <- Location
      LocationReference[n] <- BeadDNASequence[n]
      
    }
    
    # If a previous bead has the same location, match the bead identifier with the sequence from that location
    else if (Location %in% OccupiedLocations)
    {
      OccupiedLocations[n] <- OccupiedLocations[match(Location, OccupiedLocations)]
      LocationReference[n] <- LocationReference[match(Location, OccupiedLocations)]
      
    }
    
    # If the location is new, then add the new location and sequence to the list
    else
    {
      OccupiedLocations[n] <- Location
      LocationReference[n] <- BeadDNASequence[n]
      
    }
    
  }
  
  # Build a dictionary for replacements
  Reference <- BeadDNASequence
  Replacements <- LocationReference
  GroupNumberLabel <- OccupiedLocations
  
  # Build a list of all the used locations
  LocationsFound <- names(table(GroupNumberLabel))
  TotalLocations <- length(LocationsFound)
  
  # Isolate the bead identifiers and the remainder of the sequence (UMI)
  BeadIdentifiers <- subseq(ShortReadInput,start=1,end=BILength)
  Remainder <- subseq(ShortReadInput, start=BILength+1, end=nchar(ShortReadInput))
  
  # Initialize the new list of reads to use
  CollapsedBeadIdentifiers <- ShortReadInput
  
  # For each location, rewrite the bead identifier to match the representative bead identifier used
  for(Counter in 1:TotalLocations){
    
    # Find the representative (most common) bead identifier for a location
    ReplacementRepresentative <- Replacements[match(LocationsFound[Counter], GroupNumberLabel)]
    
    # Find all other bead identifiers that share that location
    ReferencestoReplaceinGroup <- Reference[GroupNumberLabel==LocationsFound[Counter]]
    
    # Find the index of those reads that have bead identifiers tracked to a location
    ReplacementIndex <- BeadIdentifiers %in% ReferencestoReplaceinGroup
    
    # Save the leftover UMI sequence of the reads that have bead identifiers tracked to a location
    RemainderSequence <- Remainder[ReplacementIndex]
    
    # Reconstruct the new reads with the common bead identifier for a location
    ReconstructedRead <- paste0(ReplacementRepresentative,RemainderSequence)
    
    # Insert those reconstructed reads back into the full data set
    CollapsedBeadIdentifiers <- DNAStringSet(replace(CollapsedBeadIdentifiers, ReplacementIndex, ReconstructedRead))
  }
  
  # Data output
  LocationCompendium <- data.frame(Reference, Replacements, GroupNumberLabel)
  
  # Data output
  output <- ShortReadQ(sread=CollapsedBeadIdentifiers, id=IDInput, quality=ShortReadQuality)
  
  write.table(as.character(unique(Replacements)), paste0(FileName, "locationbarcodes.csv"), 
              quote=FALSE, row.names=FALSE, col.names=FALSE)
  
  ShortReadGroupNumberEdit <- output
  writeFastq(output, paste0(FileName,"LocationCompressedR1.fastq.gz"))
  write.csv(LocationCompendium, paste0(FileName, "LocationCompendium.csv"))
}

#  Function to scan the barcode quality of each bead and keep or discard them based on the metrics

LocationQualityFilter <- function(CorrelationFile, GroupingFile, FileName)
{
  # Extract parameters from correlation file
  BeadIdentifier <- CorrelationFile$ColumnBeadIdentifier
  ColumnScores <- as.numeric(CorrelationFile$ColumnQualityScore)
  ColumnCounts <- as.numeric(CorrelationFile$ColumnUniqueBarcodes)
  RowScores <- as.numeric(CorrelationFile$RowQualityScore)
  RowCounts <- as.numeric(CorrelationFile$RowUniqueBarcodes)
  
  # Apply thresholds for percentage of location barcodes used by best match and total barcode reads
  MissedHits <- ColumnScores<60 | RowScores<60 | ColumnCounts<10 | RowCounts<10
  
  # Isolate beads that do not pass the thresholds
  BadBeadIdentifiers <- BeadIdentifier[MissedHits]
  
  # Replace the correlation file with the filtered correlation file
  assign(paste("TempCorrelation"), CorrelationFile[!MissedHits,])
  
  # Filter the dictionary of grouping pairs by the filtered dictionary
  LocationQualityFilter <- GroupingFile[!GroupingFile$GroupNames %in% BadBeadIdentifiers,]
  
  # Data output
  write.table(as.character(unique(LocationQualityFilter$GroupNames)), paste0(FileName, "filteredcellbarcodes.csv"), 
              quote=FALSE, row.names=FALSE, col.names=FALSE)
  
  write.csv(CorrelationFile[!MissedHits,], file = paste0(FileName, "FilteredCorrelation.csv"))
  
}


## Run analysis

# Read fastq files
Read1Filenames <- list.files(path=filefolder, pattern="*R1_001.fastq.gz$")
Read2Filenames <- list.files(path=filefolder, pattern="*R2_001.fastq.gz$")
OutputNames <- sub("_S.*$", "", Read1Filenames)

# Break if group edited already done
if(length(list.files(path=filefolder, pattern="*GroupEditedR1.fastq.gz"))!=0) { stop("GroupEditedR1 Already Complete")}

# List of sequencing runs to analyze
print(OutputNames)




# Process each sequencing run
for (i in 1:length(Read1Filenames))
{
  
  # Get the name of the file
  TempFileNameR1 <- Read1Filenames[i]
  print(TempFileNameR1)
  
  # Analyze bead identifiers and UMIs
  TempFastqR1 <- readFastq(paste(filefolder, TempFileNameR1, sep = ""))
  assign(paste(OutputNames[i], "R1", sep=""), TempFastqR1)
  
  # Save short read segment of sequencing runs
  TempShortReadR1 <- sread(TempFastqR1)
  assign(paste(OutputNames[i], "R1Reads", sep=""), TempShortReadR1)
  
  # Analyze genes and location barcodes
  TempFileNameR2 <- Read2Filenames[i]
  TempFastqR2 <- readFastq(paste(filefolder, TempFileNameR2, sep = ""))
  assign(paste(OutputNames[i], "R2", sep=""), TempFastqR2)
  TempShortReadR2 <- sread(TempFastqR2)
  assign(paste(OutputNames[i], "R2Reads", sep=""), TempShortReadR2)
  
  # Use function to provide vector of bead identifiers to match reads
  print("Extracting Bead Identifiers")
  TempBIExtraction <- BeadIdentifierandMismatchExtractionwUMIs(TempShortReadR1, FileName=OutputNames[i], 
                                                               MinimumAppearance=BeadCutoff, CutHeight=ManualEdit)
  assign(paste(OutputNames[i], "BIs", sep=""), TempBIExtraction)
  
  # Store the grouping of bead identifiers to grouped beads
  TempGrouping <- read.csv(paste0(OutputNames[i], "Grouping.csv"))
  
  # Use function to provide vector of column location barcodes to match reads
  print("Extracting Column Tags")
  TempBCExtractionColumn <- BarcodeExtraction(TempShortReadR2, BarcodesColumn, BCErrorAllowance)
  assign(paste(OutputNames[i], "ColumnBCs", sep=""), TempBCExtractionColumn)
  
  # Use function to provide vector of row location barcodes to match reads
  print("Extracting Row Tags")
  TempBCExtractionRow <- BarcodeExtraction(TempShortReadR2, BarcodesRow, BCErrorAllowance)
  assign(paste(OutputNames[i], "RowBCs", sep=""), TempBCExtractionRow)
  
  # Use function to link bead identifiers to column location barcodes
  print("Establishing Column Correlation")
  TempCorrelationColumn <- BIBarcodeCorrelation(TempBIExtraction, TempBCExtractionColumn, TempGrouping, "Column", OutputNames[i])
  assign(paste(OutputNames[i], "ColumnCorrelation", sep=""), TempCorrelationColumn)
  
  # Use function to link bead identifiers to row location barcodes
  print("Establishing Row Correlation")
  TempCorrelationRow <- BIBarcodeCorrelation(TempBIExtraction, TempBCExtractionRow, TempGrouping, "Row", OutputNames[i])
  assign(paste(OutputNames[i], "RowCorrelation", sep=""), TempCorrelationRow)
  
  # Create dictionary of bead identifiers and location barcodes
  TempCorrelation <- cbind(TempCorrelationColumn, TempCorrelationRow)
  write.csv(TempCorrelation, file = paste0(OutputNames[i],BCErrorAllowance, "ErrorsAllowedCorrelation.csv"))
  
  # Apply quality threshold to grouping dictionary
  if(QualityTrim==TRUE)
  {TempGrouping <- LocationQualityFilter(TempCorrelation, TempGrouping, FileName = OutputNames[i])}
  
  # Use bead grouping dictionary to edit bead identifiers
  print("Editing R1 Reads")
  GroupEditedR1 <- ShortReadGroupNumberEdit(TempFastqR1, TempGrouping, FileName = OutputNames[i])
  
  # Load the group edited R1 for location compression
  IntermediateGroupEditedR1 <- readFastq(paste0(OutputNames[i], "GroupEditedR1.fastq.gz"))
  
  # Use location grouping dictionary to edit bead identifiers
  print("Compressing Location R1 Reads")
  LocationEditedR1 <- LocationCompressionFunction(IntermediateGroupEditedR1, CorrelationFile = TempCorrelation, FileName = OutputNames[i])
  
}



#####################

# External pipeline using STAR and Dropseq tools commands

# Edit following line of code in Drop-seq_alignment from BASE_RANGE=1-12 to BASE_RANGE=1-11:

# # Stage 1: pre-alignment tag and trim
# tag_cells="${dropseq_root}/TagBamWithReadSequenceExtended SUMMARY=${outdir}/unaligned_tagged_Cellular.bam_summary.txt \
# BASE_RANGE=1-11 BASE_QUALITY=10 BARCODED_READ=1 DISCARD_READ=false TAG_NAME=XC NUM_BASES_BELOW_QUALITY=1 INPUT=${unmapped_bam}"

# Run following terminal commands:

# java -jar Drop-seq_tools-1.12/3rdParty/picard/picard.jar FastqToSam F1=FileLocationCompressedR1.fastq.gz F2=FileR2_001.fastq.gz o=File.bam sample_name="File"

# ./Drop-seq_tools-1.12/Drop-seq_alignment.sh -g HomoSapiens/ -r HomoSapiens/Homo_sapiens.GRCh38.dna.primary_assembly.fa -d Drop-seq_tools-1.12/ -o OutputFolder/ -p File.bam 

# ./Drop-seq_tools-1.12/DigitalExpression i=File.bam o=OutputFolder/output.dge.txt.gz summary=OutputFolder/output.dge.summary.txt cell_bc_file=locationbarcodes.csv



##################### 

# Proteome analysis

#####################

# Libraries

library(plyr)
library(tidyverse)
library(Matrix)

# If genes should be analyzed along with proteins
GeneAnalysis <- TRUE
GeneSource <- "Path containing output.dge.txt.gz and LocationCompendium"

setwd("Path containing Genepix output")

## List of chips and counts of cells and beads based on video taken in snake pattern 
## Chambers started with F1 -> A1, A2 -> F2, F3 -> A3, A4 -> F4, A5 -> F5, F6 -> A6


# FileName <- "AX204"
# Cells <- c(1,0,0,0,0,1,1,3,1,1,0,0,
#            0,5,6,2,2,2,2,1,0,1,1,3,
#            2,2,1,0,1,2,0,0,0,1,0,0)
# Beads <- c(1,1,4,0,2,2,3,8,6,4,2,5,
#            5,8,0,6,7,4,5,2,2,6,9,3,
#            5,3,1,3,1,7,3,2,1,3,2,0)
# FileName <- "AX206"
# Cells <- c(2,1,0,0,0,1,4,0,3,7,0,3,
#            7,8,6,9,0,4,1,1,1,4,1,3,
#            0,5,1,1,0,0,0,1,0,2,0,0)
# Beads <- c(5,4,4,1,3,6,4,1,3,4,5,3,
#            9,2,0,1,1,2,2,4,4,6,2,3,
#            2,6,6,2,3,2,1,4,1,1,0,3)
# FileName <- "AX207"
# Cells <- c(0,0,0,2,0,0,1,0,1,0,0,3,
#            0,0,0,0,1,2,0,0,0,0,0,0,
#            0,1,1,0,0,0,0,0,0,0,0,0)
# Beads <- c(6,9,7,2,4,7,7,5,4,2,12,3,
#            5,1,2,3,2,1,2,16,1,2,2,2,
#            2,2,2,0,2,3,0,0,2,1,6,1)
# FileName <- "AX208"
# Cells <- c(0,0,1,1,1,1,6,1,1,5,2,1,
#            5,2,0,4,1,1,0,0,1,0,0,0,
#            0,0,0,0,4,2,0,0,1,0,0,0)
# Beads <- c(0,2,1,1,1,2,0,4,4,3,7,2,
#            1,4,3,5,2,3,0,0,0,0,0,0,
#            1,7,10,1,1,2,0,0,0,0,0,0)
# FileName <- "AX218"
# Cells <- c(0,0,1,1,0,1,0,2,1,3,5,0,
#            0,8,3,4,0,2,1,2,2,4,4,3,
#            0,0,0,0,1,1,0,1,1,0,0,2)
# Beads <- c(2,1,2,0,2,1,3,6,6,11,10,0,
#            7,9,6,7,3,2,5,2,6,3,4,10,
#            5,5,3,4,4,2,1,2,1,3,3,5)
# FileName <- "AX219"
# Cells <- c(1,3,1,0,1,2,5,0,2,2,5,4,
#            2,4,2,6,2,6,0,6,0,2,4,0,
#            1,6,2,1,1,0,2,0,0,2,1,0)
# Beads <- c(1,6,8,2,4,7,9,8,4,6,2,5,
#            6,7,2,5,4,7,4,6,7,3,3,5,
#            13,5,4,4,3,4,9,8,11,9,7,3)

# List of DEAL barcode elements
Barcodes <- c("B", "C", "D")

# Read in genepix values
File <- read.xlsx(paste0(FileName,".xlsx"), sheetIndex = 2)

# List of blocks
Blocks <- unique(File$Block)

# DEAL array goes in snake pattern, creates alternating direction of antibody barcodes
Group1 <- c(1:6, 13:18, 25:30)
Group2 <- c(7:12, 19:24, 31:36)

# Depending on alignment of Genepix measurement, here are the positions of relevant proteins for the two directions of DEAL barcodes
Registry1 <- c(NA,NA,NA,NA,"B","C","D",NA,NA,NA,NA)
Registry2 <- c(NA,NA,NA,"D","C","B",NA,NA,NA,NA,NA)

# Correspondence between blocks as measured by Genepix and Cell/Bead counting vectors
A1TopRight <- c(36,25,24,13,12,1,35,26,23,14,11,2,34,27,22,15,10,3,33,28,21,16,9,4,32,29,20,17,8,5,31,30,19,18,7,6)

# Observations to save from Genepix data
KeepData <- c("Block", "Row", "F635.Mean", "F532.Mean")

# How many entries are used
Entries <- length(KeepData)

# Initialize output of protein analysis
Output <- array(dim = c(length(Blocks)*length(Barcodes), Entries))
Output <- data.frame(Output)
colnames(Output) <- KeepData

# Extract observations out of Genepix data
FileExtract <- File[,KeepData]

# Extract the identity of the proteins
FileExtract[FileExtract$Block %in% Group1,Entries+1] <- Registry1[FileExtract[FileExtract$Block %in% Group1,]$Row]
FileExtract[FileExtract$Block %in% Group2,Entries+1] <- Registry2[FileExtract[FileExtract$Block %in% Group2,]$Row]

# Clean up the extracted Genepix data
ReducedFileExtract <- na.omit(FileExtract)
colnames(ReducedFileExtract) <- c("Block", "Row", "Mean635", "Mean532", "Barcode")

# Extract fluorescence values
Values <- ReducedFileExtract[,-c(1:2,Entries+1)]

# Comb through different chambers and barcodes and collect average protein values
i=1
Output[c("Chamber", "Cells", "Beads")] <- NA

for (n in 1:length(Blocks))
{
  for (m in 1:length(Barcodes))
  {
    Output[i,1] <- Blocks[n]
    Output[i,2] <- Barcodes[m]
    Output[i,3:Entries] <- colMeans(Values[ReducedFileExtract$Block==Blocks[n] & ReducedFileExtract$Barcode==Barcodes[m],])
    i=i+1
  }
}

# Collect chamber listing based on Genepix to video order conversion
Output[,Entries+1] <- A1TopRight[as.numeric(as.character(Output[,1]))]

# Add number of cells in each chamber
Output[,Entries+2] <- Cells[as.numeric(as.character(Output[,Entries+1]))]

# Add number of beads in each chamber
Output[,Entries+3] <- Beads[as.numeric(as.character(Output[,Entries+1]))]

# Clean up Output
colnames(Output) <- c("Block", "Barcode", "Mean635", "Mean532", "Chamber", "Cells", "Beads")

# Collect chambers with no cells
ZeroCellChambers <- Output[Output["Cells"]==0,][c("Barcode", "Mean635")]

# Obtain background values of each protein value from zero cell chambers
BackgroundValues <- ddply(ZeroCellChambers, .(Barcode), numcolwise(mean))

# Diagnostic plot of protein-cell dependence
RawPlot <- ggplot(Output, aes(x=Barcode,y=Mean635,label=Block)) + 
  geom_jitter(aes(color=factor(Cells)), width=0.3, size=4, alpha=0.4) + 
  labs(title=FileName)
assign(paste0(FileName, "RawPlot"), RawPlot)

# Normalize outputs to the total scale of measurements
NormOutput <- Output

# For each protein, obtain min and max and scale measurements between
for(i in 1:length(Barcodes))
{
  Label <- Barcodes[i]
  Suboutput <- Output[which(Output$Barcode == Label),]
  Max635 <- max(Suboutput$Mean635)
  Min635 <- min(Suboutput$Mean635)
  Norm635 <- (Suboutput$Mean635-Min635)/(Max635-Min635)
  NormOutput[which(NormOutput$Barcode == Label),"Mean635"] <- Norm635
}
NormPlot <- ggplot(NormOutput, aes(x=Barcode,y=Mean635,label=Block)) + geom_jitter(aes(color=factor(Cells)), width=0.3, size=4, alpha=0.4) + labs(title=FileName)
assign(paste0(FileName, "NormPlot"), NormPlot)

# Data output
write.csv(Output, file = paste0(FileName,"Output.csv"))


## Link transcriptome data to proteomics data
if(GeneAnalysis==TRUE)
{
  # Conversion between video of cells and beads and the location barcodes order of chambers
  A1TopRightBarcodeConversion <- c("X6-FY1-1","X5-EY1-1","X4-DY1-1","X3-CY1-1","X2-BY1-1","X1-AY1-1",
                                   "X1-AY2-2","X2-BY2-2","X3-CY2-2","X4-DY2-2","X5-EY2-2","X6-FY2-2",
                                   "X6-FY3-3","X5-EY3-3","X4-DY3-3","X3-CY3-3","X2-BY3-3","X1-AY3-3",
                                   "X1-AY4-4","X2-BY4-4","X3-CY4-4","X4-DY4-4","X5-EY4-4","X6-FY4-4",
                                   "X6-FY5-5","X5-EY5-5","X4-DY5-5","X3-CY5-5","X2-BY5-5","X1-AY5-5",
                                   "X1-AY6-6","X2-BY6-6","X3-CY6-6","X4-DY6-6","X5-EY6-6","X6-FY6-6")
  
  # Go to source containing dge matrix of counts
  setwd(GeneSource)
  
  # Extract gene count matrix
  Genes <- read.table("output.dge.txt.gz", header=TRUE, row.names=1)
  
  # Clone gene list for reference
  RefGenes <- Genes
  
  # Extract location for beads
  Location_BeadRegistry <- read.csv(paste0(FileName, "LocationCompendium.csv"))
  
  # Convert bead identifiers from gene count matrix into locations
  colnames(Genes) <- Location_BeadRegistry$GroupNumberLabel[sapply(colnames(Genes), function(x) which(Location_BeadRegistry$Reference==x))]
  
  # Apply filters to genes
  MaxGenes <- apply(Genes,1,max)>5
  TotalGenes <- apply(Genes,1,sum)>15
  VarianceGenes <- apply(Genes,1,var)>2
  
  # Pull down list of all genes
  GenesSelected <- MaxGenes & VarianceGenes & TotalGenes
  
  # Normalize genes by expression within chambers
  GenesNormalizedByExpression <- Genes / apply(Genes, 2, sum)[col(Genes)]*10000
  
  # Take log of genes
  LogGenesNormalizedByExpression <- log(GenesNormalizedByExpression+1)
  
  # Normalize among genes by standard deviation
  SDNormalizedLogGenes <- LogGenesNormalizedByExpression/apply(LogGenesNormalizedByExpression,1,sd)
  
  # Use filtered genes for linear model comparison
  GenesForLinearModel <- SDNormalizedLogGenes[GenesSelected,]
  
  # Count the number of reads per chamber
  TotalReads <- colSums(Genes)
  
  # Initialize protein level extraction
  ProteinsPerBeads <- data.frame(row.names=c(Barcodes, "Cells", "Beads"))
  
  # Initialize protein list bead names by chamber location
  ProteinsPerBeads[names(Genes)] <- NA
  
  # For each bead, collect the genes and the proteins by comparing the bead sequences and locations
  for(n in 1:length(unique(Location_BeadRegistry$Replacements)))
  {
    # Check each bead
    CurrentBead <- unique(Location_BeadRegistry$Replacements)[n]
    
    # Match it to its chamber
    CurrentChamber <- Location_BeadRegistry$Chamber[match(CurrentBead,Location_BeadRegistry$Replacements)]
    
    # Extract proteins, cells, and beads
    ProteinLevels <- Output$Mean635[Output$Chamber==CurrentChamber]
    CellCount <- Output$Cells[Output$Chamber==CurrentChamber][1]
    BeadCount <- Output$Beads[Output$Chamber==CurrentChamber][1]
    
    # Find the index for the bead based on its bead identifier compared to the gene list
    ColumnNumber <- which(names(RefGenes)==CurrentBead)
    
    # Construct a table of protein list that matches the order of the gene list
    ProteinsPerBeads[ColumnNumber] <- c(ProteinLevels, CellCount, BeadCount)
  }
  
  # Find index of chambers with cells
  CellBeadPositiveBarcodes <- ProteinsPerBeads["Cells",]>0
  
  # Apply a low gene threshold to the chambers
  HitThresholdChambers <- apply(Genes,2,sum)>500
  
  # Collect chambers that pass filter
  KeptChambers <- CellBeadPositiveBarcodes & HitThresholdChambers
  
  # Collect genes from chambers with cells
  GenesWithCells <- Genes[,KeptChambers]
  
  # Subtract background values from protein measurements
  BGSubtractedProteins <- sweep(ProteinsPerBeads[Barcodes,KeptChambers],1,BackgroundValues[,2])
  rownames(BGSubtractedProteins) <- paste0(Barcodes, "minusBG")
  
  # Correct background values to prevent log-normalization problems
  BGSubtractedProteins[BGSubtractedProteins<0] <- 1
  
  # Normalize background subtracted values by the number of cells in the chamber
  CellNormalizedBGSProteins <- BGSubtractedProteins / ProteinsPerBeads["Cells",KeptChambers][col(BGSubtractedProteins)]
  rownames(CellNormalizedBGSProteins) <- paste0(Barcodes, "minusBGpercell")
  
  # Log transform proteins
  LogProteins <- log(CellNormalizedBGSProteins)
  
  # Normalize proteins by standard deviation
  SDNormalizedNoLogProteins <- CellNormalizedBGSProteins/apply(CellNormalizedBGSProteins,1,sd)
  SDNormalizedProteins <- LogProteins/apply(LogProteins,1,sd)
  
  # Convert DEAL barcode identity to protein names
  rownames(SDNormalizedProteins) <- c("PKMprotein", "CMYCprotein", "PDHK1protein")
  rownames(SDNormalizedNoLogProteins) <- c("PKMprotein", "CMYCprotein", "PDHK1protein")
  
  # Table with integrated genes and protein data for chambers with cells
  IntegratedData <- rbind(GenesWithCells, ProteinsPerBeads[,KeptChambers], BGSubtractedProteins, CellNormalizedBGSProteins, "TotalReads"=TotalReads[KeptChambers])
  
  # Table with integrated genes and protein data for all chambers
  NoCellIntegratedData <- rbind(Genes, ProteinsPerBeads, "TotalReads"=TotalReads)
  
  # Isolate single cell chambers
  SingleCellChambers <- IntegratedData["Cells",]==1
 
  # Data output
  write.csv(x=IntegratedData, file=paste0(FileName,"FullData.csv"))
  
  # Construct dataframe to perform pairwise correlation between proteins and genes
  LinearModelInitial <- rbind(GenesForLinearModel[,KeptChambers], SDNormalizedProteins)
  LinearModelTranspose <- data.frame(t(LinearModelInitial))
  LinearModelData <- (LinearModelTranspose / apply(LinearModelTranspose,2,max)[col(LinearModelTranspose)])
 
  # Initialize list to collect genes of interest
  GenesofInterest <- list()
  ProteinNames <- c()
  for (n in 1:3)
  {
    # Check each protein
    Target <- rownames(SDNormalizedProteins)[n]
    print(Target)
    ProteinNames <- c(ProteinNames, Target)
    
    # Create a matrix of linear regressions between log transfromed genes and proteins
    PairwiseMatrixLinearRegression <- apply(LinearModelData[ , 1:(ncol(LinearModelData)-3)], 2, 
                                            function(x) lm(x ~ LinearModelData[ , ncol(LinearModelData)-3+n], 
                                                           data = LinearModelData))
    assign(paste0(Target,"PairwiseLinearRegression"), PairwiseMatrixLinearRegression)
    
    # Extract coefficients and Rsquared values
    Coefficients <- sapply(PairwiseMatrixLinearRegression,coef)
    assign(paste0(Target,"Coefficients"), Coefficients)
    Rsquared <- sapply(PairwiseMatrixLinearRegression,summary)[8,,drop=FALSE]
    assign(paste0(Target,"Rsquared"), Rsquared)
    assign(paste0(FileName,Target,"LinearModel"), t(rbind(Coefficients,unlist(Rsquared))))
    
    # Perform pairwise spearman correlation
    SpearmanMatrix <- apply(LinearModelData[ , 1:(ncol(LinearModelData)-3)], 2, 
                            function(x) cor.test(x,LinearModelData[ , ncol(LinearModelData)-3+n], method="spearman"))
    assign(paste0(FileName,Target,"Spearman"), SpearmanMatrix)
    SpearmanPValues <- sapply(SpearmanMatrix, function(x) x$p.value)
    
    # Perform pairwise pearson correlation
    PearsonMatrix <- apply(LinearModelData[ , 1:(ncol(LinearModelData)-3)], 2, 
                           function(x) cor.test(x,LinearModelData[ , ncol(LinearModelData)-3+n], method="pearson"))
    assign(paste0(FileName,Target,"Pearson"), PearsonMatrix)                          
    PearsonPValues <- sapply(PearsonMatrix, function(x) x$p.value)   
    
    # Extract genes significantly correlated by the three correlation metrics
    SignificanceTable <- data.frame(cbind(Rsquared=unlist(Rsquared), SpearmanPValues, PearsonPValues))
    SignificanceTable <- cbind(SignificanceTable, RsquaredThres=SignificanceTable[,"Rsquared"]>0.4,
                               SpearmanPValuesThres=SignificanceTable[,"SpearmanPValues"]<0.10,
                               PearsonPValuesThres=SignificanceTable[,"PearsonPValues"]<0.10)
    SignificanceTable <- cbind(SignificanceTable, SoftHit=SignificanceTable[,"RsquaredThres"]|SignificanceTable[,"SpearmanPValuesThres"]|SignificanceTable[,"PearsonPValuesThres"],
                               HardHit=SignificanceTable[,"RsquaredThres"]&SignificanceTable[,"SpearmanPValuesThres"]&SignificanceTable[,"PearsonPValuesThres"])
    assign(paste0(FileName,Target,"SignificanceTable"), SignificanceTable)
    
    # Check genes significant in at least one of the three metrics
    SoftHits <- rownames(SignificanceTable[which(SignificanceTable["SoftHit"]==1),])
    names(SoftHits) <- SoftHits
    SoftHits <- list(data.frame(t(SoftHits)))
    GenesofInterest <- c(GenesofInterest, SoftHits)
  }
  
  # Compile significant gene correlations for all proteins
  GenesofInterest <- t(do.call(rbind.fill, GenesofInterest))
  colnames(GenesofInterest) <- ProteinNames
  GenesofInterest[is.na(GenesofInterest)] <- ""
  
  write.xlsx(GenesofInterest, paste0(FileName, "GenesofInterest.xlsx"), row.names = FALSE)
  
}


##################### 

# Seurat analysis

#####################


## Use libraries for aggregated Seurat analysis
library(Seurat)
library(dplyr)
library(viridis)
library(reshape2)
library(extrafont)

# Go to path containing saved gene and protein data
setwd("Data source")

# Set thresholds to beads and genes
mingeneappearancethreshold <- 5
lowUMIpercellthreshold <- 500
lowgenepercellthreshold <- 100

## For each gene data set, extract values and track
# colnames(IntegratedData) <- gsub("X", "ChipX", colnames(IntegratedData))
# colnames(NoCellIntegratedData) <- gsub("X", "ChipX", colnames(NoCellIntegratedData))
# save(list = c("IntegratedData", "NoCellIntegratedData"), file = "Chipalldata")


# Load gene data for each chip
print("Loading data")

load("Chipalldata")
Chipall <- IntegratedData
ChipNoCell <- NoCellIntegratedData

## For each protein data set, extract protein values and background measurements
# ChipVals <- data.frame(t(ProteinsPerBeads))
# rownames(ChipVals) <- gsub("X","ChipX",rownames(ChipVals))
# ChipZeros <- ChipVals[which(ChipVals[,4]==0),]
# save(list=c("Chip1Vals","Chip2Vals","Chip1Zeros","Chip2Zeros"), file = "AllProteinValues")

# Load protein data for each chip
load("AllProteinValues")

print("Applying normalization and background subtraction")

# Extract exact background values for each chip
ChipBackground <- apply(ChipZeros,2,mean)[1:3]

# Generate conversion factor based on background values from chips
ChipConversionFactors <- ChipBackground/100

# Normalize protein value by conversion factors and background values
ChipNormalizedProteins <- ChipVals
ChipNormalizedProteins[,1:3] <- t(apply(ChipVals[,1:3],1,function(x) (x-ChipBackground)/ChipConversionFactors))

# Extract gene values only from chips
Chip <- Chipall[-((nrow(Chipall)-11):nrow(Chipall)),]

# Create Seurat objects and generate metadata
print("Creating Seurat objects")

# Set gene data for chip
ChipS <- CreateSeuratObject(raw.data=Chip, project="Chip", min.cells=mingeneappearancethreshold)

# Set cell type for chip
ChipS@meta.data$celltype <- "celltype"

# Filter cells by chip
ChipS <- FilterCells(ChipS, subset.names=c("nUMI","nGene"), low.thresholds=c(lowUMIpercellthreshold,lowgenepercellthreshold), high.thresholds=c(Inf,Inf))

# Normalize, scale, and find variable genes for chip
ChipS <- NormalizeData(ChipS, display.progress=F)
ChipS <- ScaleData(ChipS, display.progress=F)
ChipS <- FindVariableGenes(ChipS, do.plot = F, display.progress=F)

# Find mitochondrial genes and add mitochondrial metrics to Seurat
mito.genes <- grep(pattern = "^MT-", x = rownames(x = ChipS@data), value = TRUE)
percent.mito <- Matrix::colSums(ChipS@raw.data[mito.genes, ])/Matrix::colSums(ChipS@raw.data)
ChipS <- AddMetaData(object = ChipS, metadata = percent.mito, col.name = "percent.mito")


## Integrate protein values in to Seurat analysis


print("Adding protein values to Seurat")

# Find raw protein values and normalize by cell number
ChipNormalizedProteins <- ChipNormalizedProteins[rownames(ChipNormalizedProteins) %in% ChipS@cell.names,]
ChipAllProts <- ChipNormalizedProteins
ChipNormalizedProteins[,1:3] <- ChipNormalizedProteins[,1:3]/ChipNormalizedProteins[,4]

# Add normalized protein values to Seurat
ChipS <- SetAssayData(ChipS, assay.type = "SCBC", slot = "raw.data", new.data = t(ChipNormalizedProteins[,1:3]))

# Add cell and bead counts to Seurat
ChipS <- AddMetaData(object = ChipS, metadata = ChipNormalizedProteins[,4:5], col.name = c("cells","beads"))

# Normalize and scale Seurat measurements
ChipS <- NormalizeData(ChipS, assay.type = "SCBC", normalization.method = "genesCLR", display.progress = F)
ChipS <- ScaleData(ChipS, assay.type = "SCBC", display.progress = F)


# Aggregate Seurat data sets
print("Integrating multiple chips")

# Attach names to chips to generate combined protein metrics
ChipNormalizedProteins[,"Chip"] <- "Chip"

# Combine protein values for aggregate data
Allprotsnormalized <- rbind(Chip1NormalizedProteins,Chip2NormalizedProteins)

# Combine raw protein values for aggregate data
Allprotsall <- rbind(Chip1AllProts,Chip2AllProts)
Allprotsall["Chip"] <- gsub(pattern = "*X(.*)", replacement="", x=gsub(pattern = "AX",replacement="A", x=rownames(Allprotsall)))
colnames(Allprotsall)[1:3] <- c("PKM2","c-MYC","PDHK1")

# Generate tidy data frame for all protein data
AllprotsallPlot <- melt(Allprotsall, id=c("Cells","Beads", "Chip"))
AllprotsallPlot[,1] <- as.factor(AllprotsallPlot[,1])

# Generate tidy data frame for all proteins including zero cell chambers
Allprotsincludingzero <- rbind(AX206Vals,AX207Vals,AX208Vals,AX218Vals,AX219Vals)[,1:4]
AllprotsincludingzeroPlot <- melt(Allprotsincludingzero, id=c(Cells))
AllprotsincludingzeroPlot[,1] <- as.factor(AllprotsincludingzeroPlot[,1])

# Generate tidy data frame for all normalized protein data
Allprotsnormalizedplot <- melt(Allprotsnormalized, id=c("Cells", "Beads", "Chip"))
Allprotsnormalizedplot$Chip <- as.factor(Allprotsnormalizedplot$Chip)
Allprotsnormalizedplot$Cells <- as.factor(Allprotsnormalizedplot$Cells)
Allprotsnormalizedplot["Celltype"] <- NA

# Add observation for protein plot to account for cell types
Allprotsnormalizedplot$Celltype[Allprotsnormalizedplot$Chip %in% c("ChipCellType1")] <- "CellType1"
Allprotsnormalizedplot$Celltype[Allprotsnormalizedplot$Chip %in% c("ChipCellType2")] <- "CellType2"

# Generate plot of all chips and protein values including number of cells
ggplot(Allprotsnormalizedplot) + 
  geom_boxplot(aes(x=variable, y=value, fill=Chip), outlier.shape = 3) +
  geom_point(aes(x=variable, y=value, fill=Chip, size=Cells), position=position_dodge(width = 0.75), alpha=0.5) +
  scale_size_discrete(range = c(1,5))

# Use Seurat tools to choose variable genes
print("Choosing variable genes")

# Extract variable genes from Seurat for individual chips
ChipSGeneNames <- ChipS@var.genes

# Combine variable genes across chips
GenestoUse <- unique(c(Chip1SGeneNames, Chip2SGeneNames))

# Choose variable genes present in each data set
GenestoUse <- intersect(GenestoUse, rownames(Chip1S@raw.data))
GenestoUse <- intersect(GenestoUse, rownames(Chip2S@raw.data))

# Merge genes from all Seurat objects
CombinedGenesbyMerge <- MergeSeurat(Chip1S, Chip2S)
CombinedGenesbyMerge <- MergeSeurat(CombinedGenesbyMerge, Chip3S)

# Add protein measurements to aggregated Seurat and normalize and scale
CombinedGenesbyMerge <- SetAssayData(CombinedGenesbyMerge, assay.type = "SCBC", slot = "raw.data", new.data = t(Allprotsnormalized[,1:3]))
CombinedGenesbyMerge <- NormalizeData(CombinedGenesbyMerge, assay.type = "SCBC", normalization.method = "genesCLR", display.progress = F)
CombinedGenesbyMerge <- ScaleData(CombinedGenesbyMerge, assay.type = "SCBC", display.progress = F)

# Analyze with PCs, tSNE
print("Analyzing combined data")

# Analyze cell metrics by all genes
CombinedGenesbyMerge@var.genes <- GenestoUse
source("BulkComp.R")
CombinedGenesbyMerge@var.genes <- TestBulkvar

# Normalize, scale, and run principal component analysis using all variable genes
CombinedGenesbyMerge <- NormalizeData(CombinedGenesbyMerge, display.progress = F)
CombinedGenesbyMerge <- ScaleData(CombinedGenesbyMerge, vars.to.regress = c("nUMI"), display.progress = F)
CombinedGenesbyMerge <- RunPCA(object = CombinedGenesbyMerge, pc.genes = CombinedGenesbyMerge@var.genes, do.print = TRUE, pcs.print = 1:5, genes.print = 5)

# Standard Seurat analysis from Satija lab Seurat vignette
VizPCA(object = CombinedGenesbyMerge, pcs.use = 1:2)
PCAPlot(object = CombinedGenesbyMerge, dim.1 = 1, dim.2 = 2, group.by = "celltype")
CombinedGenesbyMerge <- ProjectPCA(object = CombinedGenesbyMerge)
PCHeatmap(object = CombinedGenesbyMerge, pc.use = 1, do.balanced = TRUE, label.columns = FALSE)
CombinedGenesbyMerge <- JackStraw(object = CombinedGenesbyMerge, num.replicate = 50, display.progress = FALSE)
JackStrawPlot(object = CombinedGenesbyMerge, PCs = 1:20)
PCElbowPlot(object = CombinedGenesbyMerge)
CombinedGenesbyMerge <- FindClusters(object = CombinedGenesbyMerge, reduction.type = "pca", dims.use = 1:20, resolution = 1.1, print.output = 0, save.SNN = TRUE, force.recalc=TRUE)
PrintFindClustersParams(object = CombinedGenesbyMerge)
CombinedGenesbyMerge <- RunTSNE(object = CombinedGenesbyMerge, dims.use = 1:20, do.fast = TRUE)
cluster1.markers <- FindMarkers(object = CombinedGenesbyMerge, ident.1 = 1, min.pct = 0.25)
print(x = head(x = cluster1.markers, n = 5))
CombinedGenesbyMerge.markers <- FindAllMarkers(object = CombinedGenesbyMerge, only.pos = TRUE, min.pct = 0.25, thresh.use = 0.25)
CombinedGenesbyMerge.markers %>% group_by(cluster) %>% top_n(5, avg_logFC)
TSNEPlot(object = CombinedGenesbyMerge, group.by = "orig.ident", pt.size = 3)
RidgePlot(CombinedGenesbyMerge, features.plot = c("B","C","D"), nCol = 2, group.by = "celltype")

# Repeat analysis using variable genes that come from bulk data

# Import HEK and U87 public data sets
HEKBulk <- read.csv("GSE79133_NS20160217_dge_ed1.txt", sep = "", row.names = 1)
colnames(HEKBulk) <- c("HEK1", "HEK2", "HEK3")
U87Bulk <- read.csv("GSM2794663_U87_con_1_Genes_ReadCount.txt", sep = "\t", row.names=1)
colnames(U87Bulk) <- "U871"
U87Bulk1 <- read.csv("GSM2794664_U87_con_2_Genes_ReadCount.txt", sep = "\t", row.names=1)
colnames(U87Bulk1) <- "U872"
HekBulk1 <- read.csv("GSM2486332_HEK_Population.txt", sep = "", row.names=1)
HekBulk1 <- subset(HekBulk1,select=-genes)
# GSE89164
colnames(HekBulk1) <- "HEK4"
HEKBulk2 <- read.csv("gene_counts_hek.txt", sep = "", row.names = 1)
colnames(HEKBulk2) <- "HEK5"
U87Bulk2 <- read.csv("GSM2333485_U87Control1_raw_count.txt", sep = "\t")
U87Bulk2 <- U87Bulk2[-which(duplicated(U87Bulk2$external_gene_id)==TRUE),1:7]
rownames(U87Bulk2) <- U87Bulk2$external_gene_id
U87Bulk2 <- subset(U87Bulk2, select = U87Control1_raw_count)
colnames(U87Bulk2) <- "U873"

# Combine bulk data sets
GenesforBulk <- rownames(CombinedGenesbyMerge@raw.data)

# Organize bulk data
BulkDatasets <- data.frame(row.names=GenesforBulk)
BulkDatasets[colnames(HEKBulk)] <- HEKBulk[GenesforBulk,]
BulkDatasets[colnames(HekBulk1)] <- HekBulk1[GenesforBulk,]
BulkDatasets[colnames(HEKBulk2)] <- HEKBulk2[GenesforBulk,]
BulkDatasets[colnames(U87Bulk)] <- U87Bulk[GenesforBulk,]
BulkDatasets[colnames(U87Bulk1)] <- U87Bulk1[GenesforBulk,]
BulkDatasets[colnames(U87Bulk2)] <- U87Bulk2[GenesforBulk,]
BulkDatasets[is.na(BulkDatasets)] <- 0

# Create Seurat object using bulk data and normalize and scale and find variable genes
BulkS <- CreateSeuratObject(raw.data=BulkDatasets, project = "Bulks")
BulkS <- NormalizeData(BulkS, display.progress = F)
BulkS <- ScaleData(BulkS, display.progress = F)
BulkS <- FindVariableGenes(BulkS, x.low.cutoff = 0.3, x.high.cutoff = 8, y.cutoff = 1.5, do.plot = F, display.progress = F)

# Extract variable genes from bulk data
TestBulkvar <- BulkS@var.genes

###### Repeat standard Seurat analysis with selected variable genes

# Single cell tSNE analysis
CombinedGenesbyMergeSingleCells <- SubsetData(CombinedGenesbyMerge, cells.use = CombinedGenesbyMerge@meta.data$Cells==1)

CombinedGenesbyMergeSingleCells <- RunTSNE(object = CombinedGenesbyMergeSingleCells, dims.use = 1:20, do.fast = TRUE, perplexity=10)

# Repeat pairwise correlation analysis using all genes
FileName <- "AllCells"
GenesofInterest <- list()
ProteinNames <- c()
IntegratedSeuratDataset <- data.frame(as.matrix(t(rbind(CombinedGenesbyMerge@scale.data, CombinedGenesbyMerge@assay$SCBC@scale.data))))
for (n in 1:3)
{
  Target <- colnames(Allprotsnormalized[,1:3])[n]
  print(Target)

  PairwiseMatrixLinearRegression <- apply(IntegratedSeuratDataset[ , 1:(ncol(IntegratedSeuratDataset)-3)], 2, 
                                          function(x) lm(x ~ IntegratedSeuratDataset[ , ncol(IntegratedSeuratDataset)-3+n], 
                                                         data = IntegratedSeuratDataset))
  assign(paste0(Target,"PairwiseLinearRegression"), PairwiseMatrixLinearRegression)
  
  Coefficients <- sapply(PairwiseMatrixLinearRegression,coef)
  assign(paste0(Target,"Coefficients"), Coefficients)
  
  Rsquared <- sapply(PairwiseMatrixLinearRegression,summary)[8,,drop=FALSE]
  assign(paste0(Target,"Rsquared"), Rsquared)
  
  assign(paste0(FileName,Target,"LinearModel"), t(rbind(Coefficients,unlist(Rsquared))))
  
  SpearmanMatrix <- apply(IntegratedSeuratDataset[ , 1:(ncol(IntegratedSeuratDataset)-3)], 2, 
                          function(x) cor.test(x,IntegratedSeuratDataset[ , ncol(IntegratedSeuratDataset)-3+n], method="spearman"))
  assign(paste0(FileName,Target,"Spearman"), SpearmanMatrix)
  
  SpearmanPValues <- sapply(SpearmanMatrix, function(x) x$p.value)
  
  PearsonMatrix <- apply(IntegratedSeuratDataset[ , 1:(ncol(IntegratedSeuratDataset)-3)], 2, 
                         function(x) cor.test(x,IntegratedSeuratDataset[ , ncol(IntegratedSeuratDataset)-3+n], method="pearson"))
  assign(paste0(FileName,Target,"Pearson"), PearsonMatrix)                          
  
  PearsonPValues <- sapply(PearsonMatrix, function(x) x$p.value)   
  
  SignificanceTable <- data.frame(cbind(Rsquared=unlist(Rsquared), SpearmanPValues, PearsonPValues))
  SignificanceTable <- cbind(SignificanceTable, RsquaredThres=SignificanceTable[,"Rsquared"]>0.4,
                             SpearmanPValuesThres=SignificanceTable[,"SpearmanPValues"]<0.05,
                             PearsonPValuesThres=SignificanceTable[,"PearsonPValues"]<0.05)
  SignificanceTable <- cbind(SignificanceTable, SoftHit=SignificanceTable[,"RsquaredThres"]|SignificanceTable[,"SpearmanPValuesThres"]|SignificanceTable[,"PearsonPValuesThres"],
                             HardHit=SignificanceTable[,"RsquaredThres"]&SignificanceTable[,"SpearmanPValuesThres"]&SignificanceTable[,"PearsonPValuesThres"])
  assign(paste0(FileName,Target,"SignificanceTable"), SignificanceTable)
  
  SoftHits <- rownames(SignificanceTable[which(SignificanceTable["SoftHit"]==1),])
  names(SoftHits) <- SoftHits
  SoftHits <- list(data.frame(t(SoftHits)))
  GenesofInterest <- c(GenesofInterest, SoftHits)
}
library(plyr)
GenesofInterest <- t(do.call(rbind.fill, GenesofInterest))
colnames(GenesofInterest) <- ProteinNames
GenesofInterest[is.na(GenesofInterest)] <- ""
library(xlsx)
write.xlsx(GenesofInterest, paste0(FileName, "GenesofInterest.xlsx"), row.names = FALSE)
ggplot(IntegratedSeuratDataset, aes(x=B, y=ITGA10))+geom_point()+geom_smooth(method='lm',formula=y~x)

# Separately measure meta data
Metadata <- CombinedGenesbyMerge@meta.data
Metadata[,"GeneCellRatio"] <- Metadata[,1]/Metadata[,6]
Metadata[,"GeneBeadRatio"] <- Metadata[,1]/Metadata[,7]
Metadata[,"CellBeadRatio"] <- Metadata[,7]/Metadata[,6]

# Generate plots of beads and cells vs genes
ggplot(Metadata, aes(x=Beads, y=nGene)) +
  geom_point()+geom_smooth(method='lm',formula=y~x) + 
  scale_x_continuous(breaks=seq(0,11,1)) + 
  coord_fixed(ratio = 11/4000) + 
  theme(text=element_text(family="Arial"))
ggplot(Metadata, aes(x=Cells, y=nGene)) +
  geom_point()+geom_smooth(method='lm',formula=y~x) + 
  scale_x_continuous(breaks=seq(0,11,1)) + 
  coord_fixed(ratio = 9/4000) + 
  theme(text=element_text(family="Arial"))

# Violin plot of genes and UMIs
ggplot(Metadata) + 
  geom_violin(aes(x="nUMI", y=nUMI), width=0.7, fill="red") + 
  geom_jitter(aes(x="nUMI", y=nUMI), width=0.2, size=4, alpha=0.6) +
  geom_violin(aes(x="nGene", y=nGene), width=0.7) + 
  geom_jitter(aes(x="nGene", y=nGene), width=0.2, size=4, alpha=0.6) +
  theme(text=element_text(family="Calibri")) +
  labs(x = "Counts", y = "Metric")


# Extract protein value meta data including proteins, cells, beads, and total reads
NoCellIncludedMetadata <- data.frame(t(cbind(tail(Chip1NoCell,6),tail(Chip2RedoNoCell,6))))

# Plot normalized proteins across all chips
ggplot(AllprotsallPlot, aes(x=variable, y=value, color=Cells)) + 
  geom_jitter(width=0.3, size=4, alpha=0.6) +
  scale_color_manual(values=rev(viridis(9))) +
  ggtitle("Proteins") +
  ylab("Fluorescence (arbitrary units)") +
  xlab("Protein") +
  theme(legend.position = c(0.8,0.8), text=element_text(family="Arial"))

# Generate dendrograms for manual PCR replicates
library(ggdendro)

# Use data from manual PCR to compare cells between replicates
Replicate1 <- Chip1Genes[,KeptChambers]
Replicate2 <- Chip2Genes[,KeptChambers]

# Extract gene names from replicates
R1Genes <- rownames(Replicate1)
R2Genes <- rownames(Replicate2)

# Initialize comparison of replicates
ReplicateGenes <- Chip1Genes

# Label genes from replicate 1
colnames(ReplicateGenes) <- paste0(colnames(ReplicateGenes), ": 1")

# Initialize counts from replicate 2
ReplicateGenes[paste0(colnames(Chip2Genes),": 2")] <- as.integer(0)
colnames(ReplicateGenes) <- gsub("-."," ",colnames(ReplicateGenes))

# Find genes that match between replicates
ReplicateMatch <- sapply(Replicate2, function(x) match(x, Replicate1))

# Combine gene lists for replicates 1 and 2
ReplicateGenes[which(rownames(ReplicateGenes) %in% names(OldNewMatch[!is.na(OldNewMatch)==TRUE])),(ncol(OldGeneData)+1):(ncol(OldGeneData)+ncol(NewGeneData))] <- NewGeneData[!is.na(OldNewMatch),]
ReplicateGenes[(nrow(ReplicateGenes)+1):(nrow(ReplicateGenes)+nrow(NewGeneData[is.na(OldNewMatch),])),] <- 0
ReplicateGenes[(nrow(ReplicateGenes)-nrow(NewGeneData[is.na(OldNewMatch),])+1):nrow(ReplicateGenes),(ncol(OldGeneData)+1):(ncol(OldGeneData)+ncol(NewGeneData))] <- NewGeneData[is.na(OldNewMatch),]

# Write new names for genes in replicate 2 but not 1
rownames(ReplicateGenes)[(nrow(ReplicateGenes)-nrow(NewGeneData[is.na(OldNewMatch),])+1):nrow(ReplicateGenes)] <- rownames(NewGeneData[is.na(OldNewMatch),])

# Apply filters and normalizations to replicate data
VarRep <- apply(ReplicateGenes,1,var)
MaxRepGenes <- apply(ReplicateGenes,1,max)>5 
RepGenesNormalized <- ReplicateGenes / apply(ReplicateGenes,2,sum)[col(ReplicateGenes)]*10000
LogRepGenes <- log(ReplicateGenes+1)
SDNormLogRepGenes <- LogRepGenes/apply(LogRepGenes,1,sd)
RepSelectGenes <- SDNormLogRepGenes[MaxRepGenes,]

# Perform clustering on replicate data
hc <- hclust(dist(t(RepSelectGenes)))
dend <- as.dendrogram(hc)
dend_data <- dendro_data(dend, type = "rectangle")

# Color scheme for figures
NineColScheme <- c("#BF1E2D", "#f79421", "#f9ed32", "#8dc63f", "#079247", "#27aae1", "#00a79d", "#1B75BC", "#662D91", "#da1c5c")




# Perform inter-chip significance tests
AllprotsnormalizedNoRep <- rbind(Chip1NormalizedProteins,Chip2NormalizedProteins)
colnames(AllprotsnormalizedNoRep)[1:3] <- c("PKM2", "c-MYC", "PDHK1")

# Generate tidy data frames
AllprotsnormalizedplotNoRep <- melt(AllprotsnormalizedNoRep, id=c("Cells", "Beads", "Chip"))
AllprotsnormalizedplotNoRep$Chip <- as.factor(AllprotsnormalizedplotNoRep$Chip)
AllprotsnormalizedplotNoRep$Cells <- as.factor(AllprotsnormalizedplotNoRep$Cells)
AllprotsnormalizedplotNoRep["Celltype"] <- NA
AllprotsnormalizedplotNoRep$Celltype[AllprotsnormalizedplotNoRep$Chip %in% c("U87Chips")] <- "U87"
AllprotsnormalizedplotNoRep$Celltype[AllprotsnormalizedplotNoRep$Chip %in% c("HEKChips")] <- "HEK"

# Extract specific proteins
BProts <- AllprotsnormalizedplotNoRep[AllprotsnormalizedplotNoRep$variable=="PKM2",]
CProts <- AllprotsnormalizedplotNoRep[AllprotsnormalizedplotNoRep$variable=="c-MYC",]
DProts <- AllprotsnormalizedplotNoRep[AllprotsnormalizedplotNoRep$variable=="PDHK1",]

# Perform simple t.test for comparisons between cell types, or pairwise.t.test for comparisons between chips
t.test(BProts$value ~ CProts$Celltype)
t.test(CProts$value ~ CProts$Celltype)
t.test(DProts$value ~ DProts$Celltype)
pairwise.t.test(AllprotsnormalizedplotNoRep[AllprotsnormalizedplotNoRep$Celltype=="U87" & 
                                              AllprotsnormalizedplotNoRep$variable=="c-MYC",]$value, 
                AllprotsnormalizedplotNoRep[AllprotsnormalizedplotNoRep$Celltype=="U87" & 
                                              AllprotsnormalizedplotNoRep$variable=="c-MYC",]$Chip, 
                p.adjust.method = "bonferroni")



# Generate protein values and meta data
Allcellrawmeta <- rbind(Chip1Vals, Chip2Vals)
ggplot(Allcellrawmeta) + 
  geom_jitter(aes(x=Cells, y=Beads), width = 0.2, height = 0.1, alpha = 0.3)


# Random Forest classifier
# Select genes to use for training
VargenesTraining <- rownames(CombinedGenesbyMerge@scale.data) %in% GenestoUse

# Combine genes to use and protein values
MachineLearningData <- rbind(CombinedGenesbyMerge@scale.data[VargenesTraining,], CombinedGenesbyMerge@assay$SCBC@scale.data)

# Add in cell type and convert to numeric vector
CelltypeRegistry <- CombinedGenesbyMerge@meta.data$celltype
CelltypeRegistry <- gsub(x = CelltypeRegistry, "U87", "0")
CelltypeRegistry <- as.numeric(gsub(x = CelltypeRegistry, "HEK", "1"))

# Create interpretable data set for random forest R function
MachineLearningData <- rbind(MachineLearningData, CelltypeRegistry)
MachineLearningData <- t(MachineLearningData)
colnames(MachineLearningData)[ncol(MachineLearningData)] <- "CellType"

# Count number of each cell type to split training and testing set
nU87 <- length(which(MachineLearningData[,ncol(MachineLearningData)]==0))
nHEK <- length(which(MachineLearningData[,ncol(MachineLearningData)]==1))

# Choose random subset for training and testing
TrainingIndex <- c(sample(which(MachineLearningData[,ncol(MachineLearningData)]==0),nU87*3/4),sample(which(MachineLearningData[,ncol(MachineLearningData)]==1),nHEK*3/4))
TestingIndex <- setdiff(1:108, TrainingIndex)

# Generate data frames for random forest
CompleteMLDB <- as.data.frame(MachineLearningData)
colnames(CompleteMLDB) <- gsub(colnames(CompleteMLDB), pattern = "-", replacement = "")
TrainingMLDB <- CompleteMLDB[TrainingIndex,]
TestingMLDB <- CompleteMLDB[TestingIndex,]

# Generate random forest
VargenesSCBC.rf = randomForest(as.factor(CellType) ~ .,data=TrainingMLDB,importance=T,proximity=T)
VargenesSCBC.rf$importance[order(VargenesSCBC.rf$importance[,4]),]

# Generate predictions for binary cell type classification
VargenesSCBC.predtrain <- predict(VargenesSCBC.rf,type="prob")
VargenesSCBC.predtest <- predict(VargenesSCBC.rf,newdata=TestingMLDB,type="prob")
pred.train = prediction(as.vector(VargenesSCBC.predtrain[,2]),as.vector(TrainingMLDB[,c('CellType')]))
pred.train_auc = performance(pred.train, 'auc')
pred.train_rates = performance(pred.train, 'tpr','fpr')

# Find most important variables in classifier by rank
goodPredictors = rownames(VargenesSCBC.rf$importance)[order(VargenesSCBC.rf$importance[,4],decreasing=T)][1:n]

goodPredictors20 = rownames(VargenesSCBC.rf$importance)[order(VargenesSCBC.rf$importance[,4],decreasing=T)][1:20]

# Extract values for Human Protein Atlas
RNACellLines <- read.csv("rna_celline.tsv", sep = "\t")
HPAHek <- RNACellLines[RNACellLines$Sample=="HEK 293",]
HPAHekEnriched <- HPAHek[HPAHek$Gene.name %in% rownames(VargenesSCBC.rf$importance)[order(VargenesSCBC.rf$importance[,4],decreasing=T)][1:20],]

HPAU87 <- RNACellLines[RNACellLines$Sample=="U-87 MG",]
HPAU87Enriched <- HPAU87[HPAU87$Gene.name %in% rownames(VargenesSCBC.rf$importance)[order(VargenesSCBC.rf$importance[,4],decreasing=T)][1:20],]

# Collect values from SCBC data if plotting by bar graph with error bars
U87Means <- apply(TrainingMLDB[TrainingMLDB$CellType==0,goodPredictors20],2,mean)[-(which(!(goodPredictors20 %in% HPAHekEnriched$Gene.name)))]
U87SDs <- apply(TrainingMLDB[TrainingMLDB$CellType==0,goodPredictors20],2,sd)[-(which(!(goodPredictors20 %in% HPAHekEnriched$Gene.name)))]
HEKMeans <- apply(TrainingMLDB[TrainingMLDB$CellType==1,goodPredictors20],2,mean)[-(which(!(goodPredictors20 %in% HPAHekEnriched$Gene.name)))]
HEKSDs <- apply(TrainingMLDB[TrainingMLDB$CellType==1,goodPredictors20],2,sd)[-(which(!(goodPredictors20 %in% HPAHekEnriched$Gene.name)))]

# Data set for bar graph and error bars
MachineLearningComparisonDF <- data.frame(Gene=names(U87Means), U87Mean=U87Means, 
                                          U87SD=U87SDs, HEKMean=HEKMeans, HEKSD=HEKSDs, 
                                          HPAU87=HPAU87Enriched$Value[match(names(U87Means),HPAU87Enriched$Gene.name)],
                                          HPAHEK=HPAHekEnriched$Value[match(names(U87Means),HPAU87Enriched$Gene.name)])

# Extract data for box plot from SCBC
MachineLearningsComparisonDFU87 <- data.frame(melt(TrainingMLDB[TrainingMLDB$CellType==0,goodPredictors20[-(which(!(goodPredictors20 %in% HPAHekEnriched$Gene.name)))]]))
colnames(MachineLearningsComparisonDFU87) <- c("Gene", "Value")
MachineLearningsComparisonDFU87$Celltype <-  "U87"
MachineLearningsComparisonDFHEK <- data.frame(melt(TrainingMLDB[TrainingMLDB$CellType==1,goodPredictors20[-(which(!(goodPredictors20 %in% HPAHekEnriched$Gene.name)))]]))
colnames(MachineLearningsComparisonDFHEK) <- c("Gene", "Value")
MachineLearningsComparisonDFHEK$Celltype <-  "HEK"
MachineLearningsComparisonDFWhole <- rbind(MachineLearningsComparisonDFU87, MachineLearningsComparisonDFHEK)
MachineLearningsComparisonDFWhole$Gene <- factor(MachineLearningsComparisonDFWhole$Gene, levels(MachineLearningsComparisonDFHPA$Gene))

# Generate full data frame for box plot
MachineLearningsComparisonDFHPA <- data.frame(Value=c(MachineLearningComparisonDF$HPAU87, MachineLearningComparisonDF$HPAHEK), Celltype=c(rep("U87",19),rep("HEK",19)), Gene=c(as.character(MachineLearningComparisonDF$Gene),as.character(MachineLearningComparisonDF$Gene)))


# Function to obtain coefficient of variation of vector
CV <- function(x){sd(x)/mean(x)*100}


# CV Analysis generate data for genes that appear in all samples
HEKCV <- data.frame(AX207=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX207"] %>% apply(1,sum),AX208=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX208"] %>% apply(1,sum),AX208R=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX208Redo"] %>% apply(1,sum))
HEKCVNonZero <- HEKCV[apply(HEKCV, 1, function(x){all(x>0)}),]
HEKNonZeroVarGenes <- HEKCVNonZero[rownames(HEKCVNonZero) %in% GenestoUse,]
U87CV <- data.frame(AX206=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX206"] %>% apply(1,sum),AX206R=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX206Redo"] %>% apply(1,sum),AX218=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX218"] %>% apply(1,sum),AX219=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX219"] %>% apply(1,sum))
U87CVNonZero <- U87CV[apply(U87CV, 1, function(x){all(x>0)}),]
U87NonZeroVarGenes <- U87CVNonZero[rownames(U87CVNonZero) %in% GenestoUse,]
AllCV <- data.frame(AX206=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX206"] %>% apply(1,sum),AX206R=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX206Redo"] %>% apply(1,sum),AX218=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX218"] %>% apply(1,sum),AX219=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX219"] %>% apply(1,sum),AX207=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX207"] %>% apply(1,sum),AX208=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX208"] %>% apply(1,sum),AX208R=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX208Redo"] %>% apply(1,sum))
AllCVNonZero <- AllCV[apply(AllCV, 1, function(x){all(x>0)}),]
AllNonZeroVarGenes <- AllCVNonZero[rownames(AllCVNonZero) %in% GenestoUse,]
MixedComp <- data.frame(AX207=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX207"] %>% apply(1,sum),AX208=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX208"] %>% apply(1,sum),AX218=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX218"] %>% apply(1,sum),AX219=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX219"] %>% apply(1,sum))
MixedCVNonZero <- MixedComp[apply(MixedComp, 1, function(x){any(x>0)}),]
MixedNonZeroVarGenes <- MixedCVNonZero[rownames(MixedCVNonZero) %in% GenestoUse,]

# Create data frames for CV
AllCVDF <- data.frame(Sum=apply(AllCVNonZero,1,sum), CV=apply(AllCVNonZero,1,CV), Mean=apply(AllCVNonZero,1,mean))
HEKCVDF <- data.frame(Sum=apply(HEKCVNonZero,1,sum), CV=apply(HEKCVNonZero,1,CV), Mean=apply(HEKCVNonZero,1,mean))
U87CVDF <- data.frame(Sum=apply(U87CVNonZero,1,sum), CV=apply(U87CVNonZero,1,CV), Mean=apply(U87CVNonZero,1,mean))


# Repeat for genes that appear in any of the samples
HEKCVNonZero <- HEKCV[apply(HEKCV, 1, function(x){any(x>0)}),]
HEKNonZeroVarGenes <- HEKCVNonZero[rownames(HEKCVNonZero) %in% GenestoUse,]
U87CV <- data.frame(AX206=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX206"] %>% apply(1,sum),AX206R=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX206Redo"] %>% apply(1,sum),AX218=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX218"] %>% apply(1,sum),AX219=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX219"] %>% apply(1,sum))
U87CVNonZero <- U87CV[apply(U87CV, 1, function(x){any(x>0)}),]
U87NonZeroVarGenes <- U87CVNonZero[rownames(U87CVNonZero) %in% GenestoUse,]
AllCV <- data.frame(AX206=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX206"] %>% apply(1,sum),AX206R=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX206Redo"] %>% apply(1,sum),AX218=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX218"] %>% apply(1,sum),AX219=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX219"] %>% apply(1,sum),AX207=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX207"] %>% apply(1,sum),AX208=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX208"] %>% apply(1,sum),AX208R=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX208Redo"] %>% apply(1,sum))
AllCVNonZero <- AllCV[apply(AllCV, 1, function(x){any(x>0)}),]
AllNonZeroVarGenes <- AllCVNonZero[rownames(AllCVNonZero) %in% GenestoUse,]
MixedComp <- data.frame(AX207=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX207"] %>% apply(1,sum),AX208=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX208"] %>% apply(1,sum),AX218=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX218"] %>% apply(1,sum),AX219=CombinedGenesbyMerge@raw.data[,CombinedGenesbyMerge@meta.data$orig.ident=="AX219"] %>% apply(1,sum))
MixedCVNonZero <- MixedComp[apply(MixedComp, 1, function(x){any(x>0)}),]
MixedNonZeroVarGenes <- MixedCVNonZero[rownames(MixedCVNonZero) %in% GenestoUse,]
AllCVDF <- data.frame(Sum=apply(AllCVNonZero,1,sum), CV=apply(AllCVNonZero,1,CV), Mean=apply(AllCVNonZero,1,mean))
HEKCVDF <- data.frame(Sum=apply(HEKCVNonZero,1,sum), CV=apply(HEKCVNonZero,1,CV), Mean=apply(HEKCVNonZero,1,mean))
U87CVDF <- data.frame(Sum=apply(U87CVNonZero,1,sum), CV=apply(U87CVNonZero,1,CV), Mean=apply(U87CVNonZero,1,mean))


# Unoptimized Lysis buffer analysis 
AXViolinData <- read.csv("output.dge.summary.txt",sep="\t", skip=2)
AXViolinData$Chip <- "Chip"
AXViolinData$Lysis <- "LysisBuffer"

# Create data frame of all chips and plot by genes or transcripts
LysisBufferComp <- rbind(AXViolinData)
# Violin plots of cell metrics
VlnPlot(object = CombinedGenesbyMerge, features.plot = c("nGene", "nUMI", "percent.mito"), nCol = 3, group.by = "orig.ident", y.lab.rot = TRUE)

# tSNE overlay of protein values from Seurat
FeaturePlot(CombinedGenesbyMerge, features.plot = c("B","C","D"), cols.use = c("lightgrey", "blue"), pt.size = 2, nCol = 1)


# Neighbor chamber analysis
# Find 0-cell chambers and neighbors on chip
YCoord <- gsub(x=rownames(AX219Vals), ".*Y", "")
AX219Vals[order(YCoord),]
AX219Neighbors <- c(6,11,9,21,22,23,18,19,20,27,28,33,34,35,30,31,32)
AX219Vals[order(YCoord),][AX219Neighbors,]

# Extract their values
AX219NeighborAnalysis <- AX219Vals[order(YCoord),][AX219Neighbors,]

# Order correctly by position
AX219NeighborAnalysis$Position <- c(1:3,6:11,14:15,18:23)
ggplot(AX219NeighborAnalysis) +
  geom_point(aes(x=Position,y=B, size=Cells)) +
  labs(y="PKM Protein (a.u.)") +
  theme(axis.ticks.x = element_blank(), axis.text.x = element_blank())

# Isolate cells and plot by number of nearest neighbors
ZerosNeighbors <- c(2,4,6,9,10,13,14,17)
ZerosAdjacents <- c(7,6,8,4,1,2,2,1)

# Extract spearman rank order statistics
cor.test(AX219AdjacentAnalysis$Adjacents, AX219AdjacentAnalysis$B, method="spearman")

VlnPlot(object = CombinedGenesbyMerge, features.plot = c("nGene", "nUMI", "percent.mito"), nCol = 3, group.by = "orig.ident", y.lab.rot = TRUE)


##################### 

# Figure Generation

##################### 

# Proteins per chamber one chip
Output$Cells <- as.factor(Output$Cells)
RawPlot <- ggplot(Output, aes(x=Barcode,y=Mean635,label=Block)) +
  geom_jitter(aes(color=Cells), width=0.3, size=4, alpha=0.4) +
  coord_fixed(ratio=1/400) +
  scale_y_continuous(breaks = seq(0, 1500, by=300), limits=c(0,1500)) +
  scale_x_discrete(labels = c("PKM2", "c-MYC", "PDHK1")) +
  labs(x="Protein", y="Fluorescence (a.u.)") +
  theme(text=element_text(family="Arial", size = 15), legend.position = c(0.75, 0.7)) +
  scale_color_manual(values = NineColScheme)

# Beads to location compression
locationcomp <- read.xlsx(file = "SequencingLocationCompression.xlsx", sheetIndex = 2)
colnames(locationcomp) <- c("Bead","Genes", "Transcripts", "Clustering Method")
ggplot(locationcomp, aes(x=`Clustering Method`, y=Transcripts, fill=`Clustering Method`)) + 
  geom_violin(alpha = 0.4) + 
  geom_jitter(width = 0.1, show.legend= FALSE) + 
  scale_fill_manual(values=c(NineColScheme[1], NineColScheme[6])) + 
  scale_color_manual(values = c(NineColScheme[1], NineColScheme[6])) +
  theme(text = element_text(family = "Arial"), legend.position = "none") + 
  coord_fixed(ratio = 1/5000)

# Plot reads per bead by cells in chamber 
NoCellIncludedMetadata <- NoCellIncludedMetadata[NoCellIncludedMetadata$Cells<6,]
NoCellIncludedMetadata <- NoCellIncludedMetadata[NoCellIncludedMetadata$Cells<6,]
NoCellIncludedMetadata$Cells <- factor(NoCellIncludedMetadata$Cells)

ggplot(NoCellIncludedMetadata, aes(x=Cells, fill=Cells, y=TotalReads/Beads)) + 
  geom_boxplot(aes(group = Cells), alpha = 0.4, outlier.color = NA) + 
  geom_jitter(width=0.1) +
  scale_fill_manual(values = NineColScheme) +
  labs(x="Cells", y="Reads per bead") +
  theme(text=element_text(family="Arial", size = 15), legend.position = "none")

# Pairwise t tests between Cells vs Reads/Bead
pairwise.t.test((NoCellIncludedMetadataWithBeads$TotalReads/NoCellIncludedMetadataWithBeads$Beads), 
                NoCellIncludedMetadataWithBeads$Cells, p.adjust.method = "bonferroni")

# Generate pairwise t-tests between cells and reads per bead
NoCellIncludedMetadataWithBeads <- NoCellIncludedMetadata[NoCellIncludedMetadata$Beads>0,]
pairwise.t.test((NoCellIncludedMetadataWithBeads$TotalReads/NoCellIncludedMetadataWithBeads$Beads), NoCellIncludedMetadataWithBeads$Cells, p.adjust.method = "bonferroni")

# Proteins per cell by cell type
ggplot(AllprotsnormalizedplotNoRep) + 
  geom_boxplot(aes(x=variable, y=value, fill=Celltype), outlier.shape = 3, width = 0.5) +
  scale_size_discrete(range = c(1,5)) +
  scale_fill_manual(values=c(NineColScheme[1], NineColScheme[6])) + 
  theme(text = element_text(family = "Arial"), legend.position = c(0, 0.9)) + 
  coord_fixed(ratio = 1/80) +
  labs(x="Protein", y="Fluorescence (a.u.)") +
  theme(text=element_text(family="Arial", size = 15))

# TSNE plot
TSNEPlot(object = CombinedGenesbyMerge, group.by = "celltype", pt.size = 4, colors.use = c(NineColScheme[1], NineColScheme[6]), no.legend = TRUE, label.size = 10)

# Generate plot of number of chambers with X Beads and Y Cells
CountHeatmap <- data.frame(as.matrix(NoCellIncludedMetadata[,c("Cells", "Beads")] %>% table))
CountHeatmap$Cells <- as.numeric(as.character(CountHeatmap$Cells))
CountHeatmap$Beads <- as.numeric(as.character(CountHeatmap$Beads))

# Plot heatmap of beads and cells frequency
ggplot(CountHeatmap, aes(y=Cells, x=Beads, color=Freq)) +
  geom_point(size = 9) +
  scale_color_gradientn(colors = c("#FFFFFF",NineColScheme[1:5])) +
  scale_x_continuous(breaks=0:max(CountHeatmap$Beads),limits = c(0,max(CountHeatmap$Beads))) +
  scale_y_continuous(breaks = 0:max(CountHeatmap$Cells), limits = c(0,max(CountHeatmap$Cells))) +
  theme(text=element_text(family="Arial", size = 15)) +
  coord_fixed(ratio=1)
ggplot(NoCellIncludedMetadata) + geom_point(aes(x=Beads, y=TotalReads))


# Plot clustering manually for resampling
p <- ggplot(dend_data$segments) +
  geom_segment(aes(x = x, y = y, xend = xend, yend = yend), size=1, lineend = "square") +
  geom_text(family="Arial", data = dend_data$labels, aes(x, y, label = label), hjust = 1.2, angle = 45, size = 4) + 
  ylim(-8, 85) + xlim(-1,41) + labs(title="Clustering after resampling") +
  theme(text=element_text(family="Calibri"), axis.title=element_blank(), axis.text=element_blank(), axis.line = element_blank(), axis.ticks = element_blank())

# Zoomed in clustering
pinset <- ggplot(dend_data$segments) +
  geom_segment(aes(x = x, y = y, xend = xend, yend = yend), size=2, lineend = "square") +
  geom_text(family="Calibri", fontface = "bold", data = dend_data$labels, aes(x=x+0.2, y, label = label), hjust = 1.2, angle = 45, size = 7) + 
  ylim(-8, 33) + xlim(25.3,39.2) + labs(title="Clustering after resampling") +
  theme(text=element_text(family="Arial"), axis.title=element_blank(), axis.text=element_blank(), axis.line = element_blank(), axis.ticks = element_blank())


# Plot distributions of protein for all chips
ggplot(AllprotsincludingzeroPlot, aes(x=variable, y=value, color=Cells)) + 
  geom_jitter(width=0.3, size=4, alpha=0.6) +
  scale_color_manual(values=NineColScheme) +
  ggtitle("Proteins") +
  ylab("Fluorescence (arbitrary units)") +
  xlab("Protein") +
  theme(legend.position = c(0.8,0.8), text=element_text(family="Arial"))

# Compare the beads capture and sequenced on chip
LOCSCom <- read.csv("ChipLocationCompendium.csv")
load("DigitalPhysicalBeads")

# Extract beads per chamber
BeadsPutBack <- table(LOCSCom$GroupNumberLabel)

# Compare beads per chip to the Beads gathered from the video
BeadCaptureRate <- data.frame("Digital" = cbind(as.vector(BeadsPutBack), "Physical" = Beads[match(names(BeadsPutBack), A1TopRightBarcodeConversion)], "Chip" = "AX206"))
rownames(BeadCaptureRate) <- names(BeadsPutBack)

# Consolidate chip capture rate over all chips
TotalBeadCapture <- BeadCaptureRate
TotalBeadCapture <- cbind(TotalBeadCapture, BeadCaptureRate)
colnames(BeadCaptureRate) <- c("Digital", "Physical")

BeadsCapture <- TotalBeadCapture["Isolate by chip",1:2]

# Generate heat map of counts
CountHeatmap <- data.frame(as.matrix(BeadsCapture %>% table)) %>% group_by(Digital, Physical)
CountHeatmap$Digital <- as.numeric(as.character(CountHeatmap$Digital))
CountHeatmap$Physical <- as.numeric(as.character(CountHeatmap$Physical))

# Plot map for single chip and all chips
ggplot(CountHeatmap, aes(x=Physical, y=Digital, color=Freq, size=Freq))+
  geom_point()+
  scale_color_gradientn(colors = c("#FFFFFF",NineColScheme[1:5]), breaks = 1:6)+
  scale_y_continuous(breaks=0:max(CountHeatmap$Digital),limits = c(0,max(CountHeatmap$Digital)))+
  scale_x_continuous(breaks = 0:max(CountHeatmap$Physical), limits = c(0,max(CountHeatmap$Physical)))+
  theme(text=element_text(family="Arial", size = 15))+
  coord_fixed(ratio=1) +
  guides(color = guide_legend(), size = guide_legend())+
  scale_size(range=c(1,30), breaks = 1:6)
ggplot(CountHeatmap, aes(x=Physical, y=Digital, color=Freq, size=Freq))+
  geom_point()+
  scale_color_gradientn(colors = c("#FFFFFF",NineColScheme[1:5]), breaks = c(1,3,8,15,23))+
  scale_y_continuous(breaks=0:max(CountHeatmap$Digital),limits = c(0,max(CountHeatmap$Digital)))+
  scale_x_continuous(breaks = 0:max(CountHeatmap$Physical), limits = c(0,max(CountHeatmap$Physical)))+
  theme(text=element_text(family="Arial", size = 15))+
  coord_fixed(ratio=1) +
  guides(color = guide_legend(), size = guide_legend())+
  scale_size(range=c(4,18), breaks = c(1,3,8,15,23))


# Extract average of reads per bead for all combinations of cells and beads in chambers
NoCellIncludedAverages <- group_by(NoCellIncludedMetadata, Cells, Beads)
Readaveragesbychambercontents <- summarize(NoCellIncludedAverages, averages = mean(TotalReads), beadaverages = mean(TotalReads/Beads))

ggplot(Readaveragesbychambercontents, aes(x=Beads, y=Cells, size=beadaverages, color = beadaverages)) +
  geom_point() +
  scale_color_gradientn(name = "Reads per bead", colors = c(NineColScheme[1:5]), limits = c(0,3300), breaks = c(100,500,1000,2000,3000)) +
  theme(text=element_text(family="Arial", size = 15), legend.position = c(0.8, 0.7)) +
  coord_fixed(ratio=1) +
  guides(color = guide_legend(), size = guide_legend()) +
  scale_size_continuous(name = "Reads per bead", limits = c(0,3300), range = c(1,16), breaks = c(100,500,1000,2000,3000)) +
  scale_x_continuous(breaks = 0:16)

# Species mixing plot
speciesmix <- read.xlsx("AX126-1SpeciesMixingOnChip.xlsx", header = TRUE, sheetIndex = 2)
colnames(speciesmix) <- c("Bead","Human Genes","Human Transcripts", "Mouse Genes", "Mouse Transcripts")
ggplot(speciesmix) + geom_point(aes(x=`Human Transcripts`, y=`Mouse Transcripts`), size=2) + ylim(0,2800) + xlim(0,3000)+theme(text=element_text(family="Arial"))

# Plot heatmap of gene data after Seurat normalization and scaling
heatmap.2(as.matrix(ChipS@scale.data[ChipS@var.genes,]), trace = "none", margins = c(5,2), labRow = FALSE)

# Plot general sequencing metrics
ggplot(Metadata, aes(fill=celltype)) +
geom_violin(aes(x=" Genes", y=nGene), scale = "count", position = position_dodge(0.7)) +
geom_violin(aes(x=" Transcripts", y=nUMI), scale = "count", position = position_dodge(0.7)) +
geom_violin(aes(x="Mitochondrial genes", y=percent.mito*60000), scale = "count", position = position_dodge(0.7)) +
coord_fixed(ratio = 1/10000) +
scale_fill_manual(values = c(NineColScheme[1],NineColScheme[6])) +
labs(y="Counts") +
theme(text=element_text(family="Arial", size = 15), legend.position = "none") +
scale_y_continuous(limits = c(0,22500), sec.axis = sec_axis(~./600, name = "% Mitochondrial genes"))


# Protein values from bulk
ggplot(BulkProteinRatios[BulkProteinRatios$Chip=="Chip 2",]) +
  geom_col(aes(x=Protein, y=Value, group=Celltype, fill = Celltype), width = 0.5, position="dodge") +
  scale_fill_manual(values=c(NineColScheme[1],NineColScheme[6])) +
  theme(text = element_text(family = "Arial"), legend.position = c(0.1,0.9)) +
  coord_fixed(ratio = 1/2000) +
  labs(x="Protein", y="Fluorescence (a.u.)") +
  theme(text=element_text(family="Arial", size = 15))

# Proteins per cell across chips
ggplot(AllprotsnormalizedplotNoRep[AllprotsnormalizedplotNoRep$Celltype=="U87",]) +
  geom_boxplot(aes(x=variable, y=value, fill=Chip), outlier.shape = 3, width = 0.5) +
  scale_fill_manual(values=c(NineColScheme[1],NineColScheme[5],NineColScheme[6])) +
  theme(text = element_text(family = "Arial"), legend.position = "none") +
  scale_y_continuous(limits = c(-8,240)) +
  coord_fixed(ratio = 1/100) +
  labs(x="Protein", y="Fluorescence (a.u.)") +
  theme(text=element_text(family="Arial", size = 15)) 

ggplot(AllprotsnormalizedplotNoRep[AllprotsnormalizedplotNoRep$Celltype=="HEK",]) +
  geom_boxplot(aes(x=variable, y=value, fill=Chip), outlier.shape = 3, width = 0.5) +
  scale_fill_manual(values=c(NineColScheme[1],NineColScheme[6])) +
  theme(text = element_text(family = "Arial"), legend.position = "none") +
  scale_y_continuous(limits = c(-5,240)) +
  coord_fixed(ratio = 1/80) +
  labs(x="Protein", y="Fluorescence (a.u.)") +
  theme(text=element_text(family="Arial", size = 15))

# TSNE with all cells
TSNEPlot(object = CombinedGenesbyMerge, group.by = "celltype", pt.size = 4, colors.use = c(NineColScheme[1], NineColScheme[6]), no.legend = TRUE, label.size = 10)

# TSNE with all cells
TSNEPlot(object = CombinedGenesbyMerge[,CombinedGenesbyMerge@meta.data$Cells==1], group.by = "celltype", pt.size = 4, colors.use = c(NineColScheme[1], NineColScheme[6]), no.legend = TRUE, label.size = 10)

# TSNE by mitochondrial genes
TSNEPlot(object = CombinedGenesbyMerge, group.by = "percent.mito", pt.size = 4, colors.use = c(NineColScheme[1], NineColScheme[6]), no.legend = TRUE, label.size = 10)

# TSNE with protein values
FeaturePlot(CombinedGenesbyMerge, features.plot = c("D"), cols.use = c("lightgrey", NineColScheme[6]), pt.size = 4)

# Plot most important random forest variables
varImpPlot(VargenesSCBC.rf, type=2, n.var=25, main='Variable Importance for Top 10 Predictors\n(Mean decrease in Gini node impurity)')

# Plot area under curve for random forest classifiers
par(cex.axis=1.5, mar=c(5,5,1,1))
plot(pred.train_rates, main='', col='red', lwd=8, cex.lab=2, cex.axis=1.5)
pred.test = prediction(as.vector(VargenesSCBC.predtest[,2]),as.vector(TestingMLDB[,c('CellType')]))
pred.test_auc = performance(pred.test, 'auc')
pred.test_rates = performance(pred.test, 'tpr','fpr')
plot(pred.test_rates, main='', col='blue', lwd=8,add=T)
text(0.5,0.5,paste('AUC for train = ',format(pred.train_auc@y.values[[1]],digits=2,scientific=FALSE)),col="red", cex=2)
text(0.5,0.4,paste('AUC for test = ',format(pred.test_auc@y.values[[1]],digits=2,scientific=FALSE)),col="blue", cex=2)
grid()

# Plot SCBC data for machine learning classifier variables
SCBCMLPlot <- ggplot(MachineLearningsComparisonDFWhole) +
  geom_boxplot(aes(x=Gene, y=Value, fill=Celltype, color=Celltype), width = 0.6, show.legend = FALSE) +
  scale_fill_manual(values=c(NineColScheme[1],NineColScheme[6])) +
  scale_color_manual(values=c(NineColScheme[1],NineColScheme[6])) +
  ylab(label = "SCBC Values") +
  theme(text = element_text(family = "Arial")) +
  coord_flip()

# Plot Human Protein Atlas equivalent variables
HPAMLPlot <- ggplot(MachineLearningsComparisonDFHPA) +
  geom_col(aes(x=Gene, y=Value, fill=Celltype, color=Celltype), width = 0.6 ,position="dodge") +
  scale_fill_manual(values=c(NineColScheme[1],NineColScheme[6])) +
  scale_color_manual(values=c(NineColScheme[1],NineColScheme[6])) +
  coord_flip() +
  labs(y = "Human Protein Atlas") +
  theme(text = element_text(family = "Arial"), axis.title.y = element_blank(), axis.text.y = element_blank(), legend.position = c(0.5, 0.1))

grid.arrange(SCBCMLPlot, HPAMLPlot, nrow=1, widths=c(3,1))

# Plot CV
ggplot(AllCVDF)+
  geom_point(aes(x=log(Mean), y=log(CV)), alpha=0.7, color=NineColScheme[1])+
  xlim(0,8)+ylim(1,5)
ggplot(HEKCVDF)+
  geom_point(aes(x=log(Mean), y=log(CV)), alpha=0.7, color=NineColScheme[1])+
  xlim(0,8)+ylim(1,5)
ggplot(U87CVDF)+
  geom_point(aes(x=log(Mean), y=log(CV)), alpha=0.7, color=NineColScheme[1])+
  xlim(0,8)+ylim(1,5)

# Plot lysis buffer comparisons
ggplot(LysisBufferComp) + 
  geom_violin(aes(x="Chip", y=NUM_TRANSCRIPTS, color=Chip, fill=Lysis), scale = "width", lwd=1.5) + 
  theme(text=element_text(family="Arial")) +
  scale_fill_manual(values=NineColScheme[c(1,6)]) +
  scale_color_manual(values=NineColScheme[c(2,4:5,8:9)]) +
  labs(y = "UMIs Detected") +
  theme(legend.position = c(0.2, 0.7), axis.title.x = element_blank())
ggplot(LysisBufferComp) + 
  geom_violin(aes(x="Chip", y=NUM_GENES, color=Chip, fill=Lysis), scale = "width", lwd=1.5) + 
  theme(text=element_text(family="Arial")) +
  scale_fill_manual(values=NineColScheme[c(1,6)]) +
  scale_color_manual(values=NineColScheme[c(2,4:5,8:9)]) +
  labs(y = "Genes Detected") +
  theme(legend.position = c(0.2, 0.7), axis.title.x = element_blank())

# Plot tSNE with single cells
TSNEPlot(object = CombinedGenesbyMergeSingleCells, group.by = "celltype", pt.size = 4, colors.use = c(NineColScheme[1], NineColScheme[6]), no.legend = TRUE, label.size = 10)

TSNEPlot(object = CombinedGenesbyMergeSingleCells, group.by = "orig.ident", pt.size = 4, colors.use = c(NineColScheme[1:7]), label.size = 10)

# Rank nearest neighbor number and background values
ggplot(AX219AdjacentAnalysis, aes(x=Adjacents, y=B))+
  geom_smooth(method='lm',formula=y~x)+
  geom_point(size=3, color=NineColScheme[1])+
  labs(y="PKM Protein Background (a.u.)", x="Number of cells in adjacent chambers")+
  scale_x_continuous(breaks=seq(1,8))